/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**                         Copyright (c) 1996                           **
**                    by Natural MicroSystems, Inc.                     **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Natural MicroSystems, Inc.                    **
**                                                                      **
*************************************************************************/

/*
 * isupCfg.c
 *
 * Scans ascii text configuration file and downloads configuration to
 * ISUP task on specified board.
 */

/* header include files -- defines (.h) */

#ifdef UNIX
#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <stdlib.h>
#endif                                  /* UNIX */

#ifdef WIN32
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#endif                                  /* WIN32 */

#ifdef __OS2__
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#endif                                  /* __OS2__ */

#include "txcpi.h"

#include "ss7type.h"
#include "isupmgmt.h"
#include "isupver.h"
#include "cfgscan.h"
#include "txcpi.h"

extern int CDebug;
/* OS-specific definitions */
#define TRUE	1
#define FALSE	0

/* UNIX */

/* forward references */
long scanConPCs(U32 *pCpcList, U16 *numconpc, long tlast);

short GenCfg(void);
short SapCfg(void);
short NSapCfg(unsigned short board);
short CircCfg(void);
short RteCfg(void);

void showHex(unsigned char *Buf, unsigned short ByteCnt);

/* global variables */
short           debug = 0;      /* prints each packet sent/received */
S32             temp;           /* temp used by scanner macros */
char            buf[80];
S16 lastSwitchType;
U8  boardNum = 1;                  /* default board is 1 */
U16 MaxSaps;
U16 MaxNetSaps;
U16 MaxRoutes;
U32 MaxCircuits;
U16 MaxCircGrp;

/* global buffers used for netmgr requests & responses */

/* limits from general configuration section */

char configFile[256] = "ISUP.cfg";
static char *usage = "Usage: isupcfg [ -b <board number> ] [-f <filename>]\n";


/* Handy macro to get & range check a long, print a warning if its out
 * of range, and copy the value (possibly cast to a smaller type)
 * to "dest" if its in range; assumes "dest" is previously set to
 * default value. "FROM" version scans from a known position in file
 * (rather than start of file) and returns current position.
 */
#define GET_CHECK_INT(name, min, max, dest, cast)         \
{                                                           \
    if (getcfgval(name, CFGT_INT, &temp, 0) > 0) {          \
        if (temp < min || temp > max) {                   \
        fprintf(stderr,                                    \
                "Warning: %s [%ld] out of range, setting to default [%d]\n",\
                name, temp, dest);                         \
        } else {                                            \
            dest = (cast) temp;                             \
        }                                                   \
    }                                                       \
}

#define GET_CHECK_INT_FROM(name, min, max, dest, cast, from, next)    \
{                                                                       \
    if ((next = getcfgval(name, CFGT_INT, &temp, from)) > 0) {          \
        if (temp < min || temp > max) {                               \
        fprintf(stderr,                                                \
                "Warning: %s [%ld] out of range, setting to default [%d]\n",\
                name, temp, dest);                                     \
        } else {                                                        \
            dest = (cast) temp;                                         \
        }                                                               \
    }                                                                   \
}

/* Handy macro to get an address
 */
#define GET_ADDRESS(name, dest)                                       \
{                                                           			\
	char temp[CFGT_MAXSTR + 1];											\
																		\
	if (getcfgval(name, CFGT_STRING, temp, 0) > 0)             			\
	{                                                                   \
		(dest).length = strlen(temp);									\
		strncpy((S8 *)(dest).string, temp, (dest).length);                    \
	}                                                                   \
}

#define GET_ADDRESS_FROM(name, dest, from, next)                      \
{                                                                       \
	char temp[CFGT_MAXSTR + 1];                                         \
																		\
    if ((next = getcfgval(name, CFGT_STRING, temp, from)) > 0)          \
	{                                                                   \
		(dest).length = strlen(temp);									\
		strncpy((S8 *)(dest).string, temp, (dest).length);                    \
	}                                                                   \
	else																\
		(dest).length = 0;												\
}

/* Handy macro to scan the config file for a timer value and set the
 * enabled/disabled status based on the timer value (0 == disabled).
 * Assumes timer value is set to default before calling.
 */
#define GET_TIMER(name, timer)                            \
{                                                           \
  if (getcfgval(name, CFGT_INT, &temp, 0) > 0)              \
    timer.value = (U16) temp;                               \
  timer.enable = (timer.value != 0) ? 1 : 0;                \
}

#define GET_TIMER_FROM(name, timer, from, next)               \
{                                                               \
  if ((next = getcfgval(name, CFGT_INT, &temp, from)) > 0)      \
    timer.value = (U16) temp;                                   \
  timer.enable = (timer.value != 0) ? 1 : 0;                    \
}

/*
  func to dump data in hex+ascii format

IN:  Buf        data to show
	 ByteCnt    #Bytes in Buf
OUT: <file>      gets multiple lines
*/
void showHex(unsigned char *Buf, unsigned short ByteCnt)
{
		unsigned short Li, LMax, ToDo, Bi, Ai;
		char Ascii[80];
		unsigned char B;

/* Dump to std out in hex followed by ascii */

for (ToDo = ByteCnt, Bi = 0; ToDo > 0; ToDo -= LMax)
	{LMax = (ToDo > 16) ? 16 : ToDo;            /* max bytes this line */
	 printf("%04X| ", Bi);
	 Ai = 0;
	 for (Li = 0; Li < LMax; Li++)              /* each byte this line */
		 {B = *Buf++;                           /* next byte to show */
		  printf("%02X ", B);            		/* as hex */
		  if (!isprint(B))                      /* printable ascii? */
			 B = '.';                           /* no, use this */
		  Ascii[Ai++] = B;                      /* into ascii portiob */
		 }	/* for Li */
	 for (Li = LMax; Li < 16; Li++)             /* pad short ln if necessary */
		 printf("   ");                 		/* so asci lines up */
	 Ascii[Ai] = 0;                             /* terminate the ascii */
	 printf("    ");
	 printf("%s\n", Ascii);                     /* show the ascii */
	 Bi += LMax;
	}	/* for ToDo */

}	/* ShowHex */

/*
 * scanConPCs
 *
 * Build a list of concerned point codes by scanning the current section of the
 * configuration file.
 *
 * Parameters
 *
 * Returns:     next position in config file after last scanned route
 */
long scanRoute(IsupRouteCfg *route, S32 tlast)
{
    S32    tnext;
    S32    pcpri[2];


	GET_ADDRESS_FROM("ROUTE", route->address, tlast, tnext);

    if (tnext <=0)
		return(tnext);

	tlast = tnext;
	if (getcfgval("LOAD_SHARE", CFGT_BOOL, &temp, tlast) <= 0)
		temp = 0;

	route->loadShare = (Bool)temp;

    /* scan for up to MAXDPC pointcode,priority pairs */

	route->numDstPointCode = 0;

    while((tnext = getcfgval("DPC", CFGT_PCPRI, pcpri, tlast)) > 0)
	{
		route->numDstPointCode++;
		tlast = tnext;
        if (route->numDstPointCode >= MAXDPC)
		{
            fprintf(stderr, "Warning: Destination point code "
			                  "%d.%d.%d exceeds maximum [%d]\n",
                              ((pcpri[0]) >> 16) & 0xFF,
							  ((pcpri[0]) >> 8) & 0xFF,
                              (pcpri[0]) & 0xFF, MAXDPC);
        }
		else
		{
			route->dst[route->numDstPointCode - 1].dstPointCode = (U32)pcpri[0];
            route->dst[route->numDstPointCode - 1].priority     = (U8)pcpri[1];
        }
    }   /* while */

   	tnext = getcfgval("END", CFGT_INT, &temp, tlast);

    return(tnext);
}  /* scanRoute */


/*
* isupGenCfg
*
* Configure general parameters for isup
*/
short GenCfg(void)
{
   IsupGenParms cfg;
   U16        status;
   U32        opc;

    if (getcfgval("OPC", CFGT_PC, &temp, 0) <= 0)
    {
        printf("ISUP Gen Config: OPC not specified\n");
        return(-1);
    }
    opc = (U32) temp;

   /* Initialize configuration block to defaults */

   isupInitGenCfg(&cfg, opc);

   /* scan for overrides from config file */
   GET_CHECK_INT("MAX_SAPS",     1, 255,   cfg.maxSaps,     U16);
   GET_CHECK_INT("MAX_NSAPS",    1, 255,   cfg.maxNetSaps,  U16);
   GET_CHECK_INT("MAX_CIRCUITS", 0, 2147483647, cfg.maxCircuits, U32);
   GET_CHECK_INT("MAX_GROUPS",   0, 65535, cfg.maxCircGrp,  U16);
   GET_CHECK_INT("MAX_CALLREFS", 0, 65535, cfg.maxCallRefs, U32);
   GET_CHECK_INT("TXMON_TIMEOUT", 0, 65535, cfg.txmonTimeout, U32);
   GET_CHECK_INT("TIMER_RES",       1, 10,   cfg.timerRes,     S16);
   GET_CHECK_INT("UPPER_LIMIT",     0, 10,   cfg.poolUpper,     S16);
   GET_CHECK_INT("LOWER_LIMIT",     0, 10,   cfg.poolLower,     S16);

   if (getcfgval("IG_PASS_ALNG", CFGT_BOOL, &temp, 0) > 0)
      if (temp == TRUE)
            cfg.igPassAlng = TRUE;
   if (getcfgval("EXT_ELMTS", CFGT_BOOL, &temp, 0) > 0)
      if (temp == TRUE)
            cfg.extElmts = TRUE;
   if (getcfgval("RAW_MSGS", CFGT_BOOL, &temp, 0) > 0)
      if (temp == TRUE)
            cfg.rawMsgs = TRUE;
   if (getcfgval("ITU_UCICS", CFGT_BOOL, &temp, 0) > 0)
      if (temp == TRUE)
            cfg.ituUCICs = TRUE;
   if (getcfgval("ONE_GRPMSG", CFGT_BOOL, &temp, 0) > 0)
      if (temp == TRUE)
            cfg.oneGrpMsg = TRUE;

    /* SS7 4.0 new parameters */
    if (getcfgval("RMTUSRUNAVL", CFGT_BOOL, &temp, 0) > 0)
      if (temp == TRUE)
            cfg.rmtUsrUnavl     = TRUE;
    if (getcfgval("GRPRESETEVENT", CFGT_BOOL, &temp, 0) > 0)
      if (temp == TRUE)
            cfg.grpResetEvent   = TRUE;
    if (getcfgval("SLSFROMCICS", CFGT_BOOL, &temp, 0) > 0)  /* default is true, so only change if false */
      if (temp == FALSE)
            cfg.slsFromCICs     = FALSE;
    if (getcfgval("DSBLRMTUSRUNAVL", CFGT_BOOL, &temp, 0) > 0)
      if (temp == TRUE)
            cfg.dsblRmtUsrUnavl = TRUE;
    if (getcfgval("RESTARTT7", CFGT_BOOL, &temp, 0) > 0)
      if (temp == TRUE)
            cfg.restartT7       = TRUE;
    if (getcfgval("DISABLEACL", CFGT_BOOL, &temp, 0) > 0)
      if (temp == TRUE)
            cfg.disableACL      = TRUE;


    if (getcfgval("TRACE_EVENT", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= EVENT_TRACE;
        else
            cfg.traceFlags &= ~EVENT_TRACE;
    }
    if (getcfgval("TRACE_DATA", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= TRACE_DATA;
        else
            cfg.traceFlags &= ~TRACE_DATA;
    }
    if (getcfgval("TRACE_WARNING", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= ENUM_WARNING;
        else
            cfg.traceFlags &= ~ENUM_WARNING;
    }
    if (getcfgval("TRACE_ERROR", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= ENCODE_ERROR;
        else
            cfg.traceFlags &= ~ENCODE_ERROR;
    }
    if (getcfgval("TRACE_ELEMENT", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= INIT_ELMT_DBG;
        else
            cfg.traceFlags &= ~INIT_ELMT_DBG;
    }
    if (getcfgval("TRACE_TOKEN", CFGT_BOOL, &temp, 0) > 0)
    {
        if (temp)
            cfg.traceFlags |= INIT_TKN_DBG;
        else
            cfg.traceFlags &= ~INIT_TKN_DBG;
    }
   GET_TIMER("T18_TIMER",   cfg.t18);
   GET_TIMER("T19_TIMER",   cfg.t19);
   GET_TIMER("T20_TIMER",   cfg.t20);
   GET_TIMER("T21_TIMER",   cfg.t21);
   GET_TIMER("T22_TIMER",   cfg.t22);
   GET_TIMER("T23_TIMER",   cfg.t23);
   GET_TIMER("T28_TIMER",   cfg.t28);
   GET_TIMER("TGRES_TIMER", cfg.tGrpReset);

#ifdef SW_ANS88
   GET_TIMER("TFGR_TIMER",  cfg.tFrstGrpRx);
#endif

   GET_ADDRESS("CLLINAME", cfg.comLLID);


   /* save some globals for use by other config sections */
    MaxSaps     = cfg.maxSaps;
    MaxNetSaps  = cfg.maxNetSaps;
    MaxRoutes   = cfg.maxRoutes;
    MaxCircuits = cfg.maxCircuits;
    MaxCircGrp  = cfg.maxCircGrp;

   GET_CHECK_INT("QCONGONSET1",   0, 65535, cfg.qCongOnset1,  U16);
   GET_CHECK_INT("QCONGABATE1",   0, 65535, cfg.qCongAbate1,  U16);
   GET_CHECK_INT("QCONGONSET2",   0, 65535, cfg.qCongOnset2,  U16);
   GET_CHECK_INT("QCONGABATE2",   0, 65535, cfg.qCongAbate2,  U16);
   GET_CHECK_INT("QCONGONSET3",   0, 65535, cfg.qCongOnset3,  U16);
   GET_CHECK_INT("QCONGABATE3",   0, 65535, cfg.qCongAbate3,  U16);

   GET_CHECK_INT("MCONGONSET1",   0, 65535, cfg.mCongOnset1,  U16);
   GET_CHECK_INT("MCONGABATE1",   0, 65535, cfg.mCongAbate1,  U16);
   GET_CHECK_INT("MCONGONSET2",   0, 65535, cfg.mCongOnset2,  U16);
   GET_CHECK_INT("MCONGABATE2",   0, 65535, cfg.mCongAbate2,  U16);
   GET_CHECK_INT("MCONGONSET3",   0, 65535, cfg.mCongOnset3,  U16);
   GET_CHECK_INT("MCONGABATE3",   0, 65535, cfg.mCongAbate3,  U16);

#ifdef CFGTEST
        printf("General Configuration\n");
        showHex((unsigned char *)&cfg, sizeof(cfg));
#else

   /* Call the Config Request */
   status = isupGenCfg(boardNum, &cfg);
   if (status == CPI_TIMEOUT)
   {
        /* for general configuration, try one more time to avoid race
         * condition that ISUP isn't quite running yet when first
         * packet was sent, most likely to happen if ISUP on host.
         */
        status = isupGenCfg(boardNum, &cfg);
   }
   if (status == CPI_TIMEOUT)
   {
        printf("ISUP Gen Config: No response from ISUP [%d]\n", boardNum);
        return(status);
   }
   else if (status != ISUP_MGMT_SUCCESS)
   {
        printf("ISUP Gen Config: Error %d\n", status);
        return(status);
   }

#endif                               /* ! CFGTEST */

   return(0);

} /* isupGenCfg */

/*
* isupSapCfg
*
* Configure ISUP upper saps
*/
short SapCfg(void)
{
   IsupUSapCfg  cfg;
   short        status;
   S32          last, next;
   U16          sapno;
   U16          switchType = ISUP_SW_ANS92;


    for(last = 0; ;)
  	{
   		memset(&cfg, 0, sizeof(cfg));

      next = getcfgval("USER_SAP", CFGT_INT, &temp, last);
      sapno = (U16) temp;
      if (next <= 0)
        break;        /* no more user SAPs */

      last = next;                        /* save starting line in config file */

      if (sapno > MaxSaps)
      {
          fprintf(stderr, "User SAP number %d exceeds maximum [%d]\n", sapno, MaxSaps);
      		continue;
    	}

    	/* initialize user SAP defaults */


   		/* fetch overides from config file */
      if (getcfgval("SWITCH_TYPE", CFGT_STRING, buf, last) > 0)
      {
            if (strcmp(buf, "ITU") == 0)
                switchType = ISUP_SW_ITU;
            else if (strcmp(buf, "ITUWHITE") == 0)
                switchType = ISUP_SW_ITU;
            else if (strcmp(buf, "ITUBLUE") == 0)
                switchType = ISUP_SW_ITUBLUE;
            else if (strcmp(buf, "Q767") == 0)
                switchType = ISUP_SW_Q767;
            else if (strcmp(buf, "ANSI88") == 0)
                switchType = ISUP_SW_ANS88;
            else if (strcmp(buf, "ANSI92") == 0)
                switchType = ISUP_SW_ANS92;
            else if (strcmp(buf, "ANSI95") == 0)
                switchType = ISUP_SW_ANS95;
            else if (strcmp(buf, "JNTT") == 0)
                switchType = ISUP_SW_JNTT;
            else if (strcmp(buf, "ITU97") == 0)
                switchType = ISUP_SW_ITU97;
            else if (strcmp(buf, "ETSIV2") == 0)
                switchType = ISUP_SW_ETSIV2;
            else if (strcmp(buf, "ETSIV3") == 0)
                switchType = ISUP_SW_ETSIV3;
            else if (strcmp(buf, "BICC") == 0)
                switchType = ISUP_SW_BICC;
            else
                fprintf(stderr, "Warning: SAP %d unknown switch type [%s] "
                                "ignored, defaulting to ANSI92\n", sapno, buf);
    	}

      isupInitUSapCfg(&cfg, switchType);

      if (getcfgval("DATABASE", CFGT_STRING, buf, last) > 0)
            strncpy((char *)cfg.database, buf, MAX_DB_LEN);

      if (getcfgval("MASK", CFGT_STRING, buf, last) > 0)
      {
            if (strlen(buf) > 20)
            {
                fprintf(stderr, "Warning: SAP %d invalid mask, wild card routing disabled.\n", sapno);
                cfg.wildCardRoute = FALSE;
            }
            else
            {
                unsigned short  i;

                for (i = 0; i < strlen(buf); i++)
                {
                    if ((buf[i] != '0') && (buf[i] != '1'))
                    fprintf(stderr, "Warning: SAP %d invalid mask, wild card routing disabled.\n", sapno);
                    cfg.wildCardRoute = FALSE;
                }

                strncpy((char *)cfg.wildCardMask, buf, 20);
                cfg.wildCardRoute = TRUE;
            }
    	}
      else
            cfg.wildCardRoute = FALSE;

      if ((next = getcfgval("MAX_USER2USER", CFGT_INT, &temp, last)) > 0)
            cfg.maxMsgLength    = (U8)temp;

      GET_TIMER("T1_TIMER", cfg.t1);
      GET_TIMER("T2_TIMER", cfg.t2);
      GET_TIMER("T5_TIMER", cfg.t5);
      GET_TIMER("T6_TIMER", cfg.t6);
      GET_TIMER("T7_TIMER", cfg.t7);
      GET_TIMER("T8_TIMER", cfg.t8);
      GET_TIMER("T9_TIMER", cfg.t9);
      GET_TIMER("T16_TIMER", cfg.t16);
      GET_TIMER("T17_TIMER", cfg.t17);
      GET_TIMER("T31_TIMER", cfg.t31);
      GET_TIMER("T33_TIMER", cfg.t33);
      GET_TIMER("TEX_TIMER", cfg.tEx);
      GET_TIMER("T27_TIMER", cfg.t27);
      GET_TIMER("T34_TIMER", cfg.t34);
      GET_TIMER("TCCR_TIMER", cfg.tCCR);
      GET_TIMER("TCRM_TIMER", cfg.tCRM);
      GET_TIMER("TCRA_TIMER", cfg.tCRA);

#ifdef CFGTEST
      printf("User SAP Configuration\n");
      showHex((unsigned char *)&cfg, sizeof(cfg));
#else

   		/* Call the Config Request */
      status = isupUSapCfg(boardNum, &cfg, sapno);
      if (status == CPI_TIMEOUT)
        {
            printf("ISUP USAP Config: No response from board\n");
            return(status);
        }
      else if (status != ISUP_MGMT_SUCCESS)
        {
            printf("ISUP USAP Config: Error %d\n", status);
            return(status);
        }

#endif                               /* ! CFGTEST */

   		/* skip up to end of section */
   		next = getcfgval("END", CFGT_INT, &temp, last);
  		last = next;
    }   /* end for loop */

    return(0);

} /* isupSapCfg */

/*
* isupNSapCfg
*
* Configure lower saps with mtp3
*/
short NSapCfg(unsigned short board)
{
   IsupNSapCfg  cfg;
   U16          switchType = ISUP_SW_ANS92;
   short        status;
   S32          last, next;
   U16          nsapno;

   for (last = 0; ;)
   {
        if ((next = getcfgval("NSAP", CFGT_INT, &temp, last)) <= 0)
            break;                          /* no more NSAPs */

        last = next;                        /* save starting line in config file */
        nsapno = (U16) temp;
        if (nsapno > MaxNetSaps)
        {
          fprintf(stderr, "NSAP number %d exceeds maximum [%d]\n", nsapno, MaxNetSaps);
          return(0);
        }

        if (getcfgval("SWITCH_TYPE", CFGT_STRING, buf, last) > 0)
        {
            if (strcmp(buf, "ITU") == 0)
                switchType = ISUP_SW_ITU;
            else if (strcmp(buf, "ITUWHITE") == 0)
                switchType = ISUP_SW_ITU;
            else if (strcmp(buf, "ITUBLUE") == 0)
                switchType = ISUP_SW_ITUBLUE;
            else if (strcmp(buf, "Q767") == 0)
                switchType = ISUP_SW_Q767;
            else if (strcmp(buf, "ANSI88") == 0)
                switchType = ISUP_SW_ANS88;
            else if (strcmp(buf, "ANSI92") == 0)
                switchType = ISUP_SW_ANS92;
            else if (strcmp(buf, "ANSI95") == 0)
                switchType = ISUP_SW_ANS95;
            else if (strcmp(buf, "JNTT") == 0)
                switchType = ISUP_SW_JNTT;
            else if (strcmp(buf, "ITU97") == 0)
                switchType = ISUP_SW_ITU97;
            else if (strcmp(buf, "ETSIV2") == 0)
                switchType = ISUP_SW_ETSIV2;
            else if (strcmp(buf, "ETSIV3") == 0)
                switchType = ISUP_SW_ETSIV3;
            else if (strcmp(buf, "BICC") == 0)
                switchType = ISUP_SW_BICC;
            else
                fprintf(stderr, "Warning: NSAP %d unknown switch type [%s]"
                                "ignored, defaulting to ANSI92\n", nsapno, buf);
        }
        lastSwitchType = switchType;

    /* initialize nsap to defaults */

        isupInitNSapCfg(&cfg, switchType);

        if (getcfgval("DATABASE", CFGT_STRING, buf, last) > 0)
            strncpy((char *)cfg.database, buf, MAX_DB_LEN);

        if (getcfgval("SSF", CFGT_STRING, buf, last))
        {
            if (strcmp(buf, "INTERNATIONAL") == 0)
                cfg.ssf = ISUP_SSF_INTL;
            else if (strcmp(buf, "0") == 0)
                cfg.ssf = ISUP_SSF_INTL;
            else if (strcmp(buf, "1") == 0)
                cfg.ssf = ISUP_SSF_SPARE;
            else if (strcmp(buf, "2") == 0)
                cfg.ssf = ISUP_SSF_NAT;
            else if (strcmp(buf, "3") == 0)
                cfg.ssf = ISUP_SSF_RES;
            else if (strcmp(buf, "NATIONAL") == 0)
                cfg.ssf = ISUP_SSF_NAT;
            else if (strcmp(buf, "RESERVED") == 0)
                cfg.ssf = ISUP_SSF_RES;
            else if (strcmp(buf, "SPARE") == 0)
                cfg.ssf = ISUP_SSF_SPARE;
            else
                fprintf(stderr, "Warning: NSAP %d unknown SSF type [%s]"
                                "ignored, defaulting to NATIONAL\n", nsapno, buf);
        }
        if ((next = getcfgval("MTP_SAP", CFGT_INT, &temp, last)) > 0)
            cfg.spId = (U16)temp;
        else
            cfg.spId = nsapno;

        if ((next = getcfgval("MTP_INSTANCE", CFGT_INT, &temp, last)) > 0)
        {
            if( temp > MAXBOARD )
            {
                fprintf(stderr, "Warning: NSAP %d MTP_INSTANCE [%d] out of range 0..%d,"
                                "defaulting to %d\n", nsapno, temp, MAXBOARD, boardNum);
            }
            else
                cfg.dstInstance = (U8)temp;
        }

        if ((next = getcfgval("NUMRESIND", CFGT_INT, &temp, last)) > 0)
        {
            cfg.numResInd = (U8)temp;
        }
    #ifdef CFGTEST
            printf("Network SAP Configuration\n");
            showHex((unsigned char *)&cfg, sizeof(cfg));
    #else

    /* Call the Config Request */
            status = isupNSapCfg(boardNum, &cfg, nsapno);
            if (status == CPI_TIMEOUT)
            {
                printf("ISUP NSAP Config: No response from board\n");
                return(status);
            }
            else if (status != ISUP_MGMT_SUCCESS)
            {
                printf("ISUP NSAP Config: Error %d\n", status);
                return(status);
            }

    #endif                               /* ! CFGTEST */

    /* skip up to end of section */
    next = getcfgval("END", CFGT_INT, &temp, last);
    last = next;
   }
   return(0);
}  /* isupNSapCfg */

/*
* isupCircCfg
*
* Configure Address Table
*/
short CircCfg(void)
{
   IsupCircCfg  cfg;
   short        status;
   S32          last, next;
   S16          switchType = lastSwitchType;

    /* initialize circuit defaults */

    for(last = 0; ;)
    {
        U32     cic;
        U32     dpc;
        U32     circuit;
        U32     cirCnt;

        next = getcfgval("CIRCUIT", CFGT_INT, &temp, last);
        circuit = (U32) temp;

        if (next <= 0)
          break;                          /* no more user SAPs */

        last = next;                        /* save starting line in config file */

        if ((next = getcfgval("DPC", CFGT_PC, &temp, last)) <= 0)
        {
            printf("Error: CIRCUIT %ld missing DPC\n", circuit);
            break;
        }
        dpc = (U32) temp;

        isupInitCircCfg(&cfg, dpc);

        if ((next = getcfgval("ALT_OPC", CFGT_PC, &temp, last)) > 0)  /* altOPC */
        {
            cfg.altOrgPointCode = (U32) temp;
        }

        next = getcfgval("CIC", CFGT_INT, &temp, last);
        if (next <= 0)
          cic = 1;
        else
          cic = (U32) temp;

        if ((next = getcfgval("NUM_CIRCUITS", CFGT_INT, &temp, last)) <= 0)
            cirCnt = 1;
        else
            cirCnt = (U32)temp;


        if (getcfgval("SWITCH_TYPE", CFGT_STRING, buf, last) > 0)
        {
            if (strcmp(buf, "ITU") == 0)
                switchType = ISUP_SW_ITU;
            else if (strcmp(buf, "ITUWHITE") == 0)
                switchType = ISUP_SW_ITU;
            else if (strcmp(buf, "ITUBLUE") == 0)
                switchType = ISUP_SW_ITUBLUE;
            else if (strcmp(buf, "Q767") == 0)
                switchType = ISUP_SW_Q767;
            else if (strcmp(buf, "ANSI88") == 0)
                switchType = ISUP_SW_ANS88;
            else if (strcmp(buf, "ANSI92") == 0)
                switchType = ISUP_SW_ANS92;
            else if (strcmp(buf, "ANSI95") == 0)
                switchType = ISUP_SW_ANS95;
            else if (strcmp(buf, "JNTT") == 0)
                switchType = ISUP_SW_JNTT;
            else if (strcmp(buf, "ITU97") == 0)
                switchType = ISUP_SW_ITU97;
            else if (strcmp(buf, "ETSIV2") == 0)
                switchType = ISUP_SW_ETSIV2;
            else if (strcmp(buf, "ETSIV3") == 0)
                switchType = ISUP_SW_ETSIV3;
            else if (strcmp(buf, "BICC") == 0)
                switchType = ISUP_SW_BICC;
            else
                fprintf(stderr, "Warning: CIRCUIT %ld unknown switch type [%s]"
                                "ignored, defaulting to ANSI92\n", circuit, buf);
        }
        cfg.switchType = switchType;
        switch ( cfg.switchType )
        {
          case ISUP_SW_JNTT: /* Make ITU adjustments */
          case ISUP_SW_ITU:
          case ISUP_SW_ITUBLUE:
          case ISUP_SW_Q767:
          case ISUP_SW_ITU97:
          case ISUP_SW_ETSIV2:
          case ISUP_SW_ETSIV3:
            cfg.t12.value     = 15;
            cfg.t13.value     = 300;
            cfg.t14.value     = 15;
            cfg.t15.value     = 300;
            break;
          default:
            break;
        }

        if ((next = getcfgval("CIRCUIT_TYPE", CFGT_STRING, buf, last)) > 0)
        {
          if (!strcmp("INCOMING", buf))
                cfg.circuitType   = ISUP_CIR_INCOMING;
          else if (!strcmp("OUTGOING", buf))
                cfg.circuitType   = ISUP_CIR_OUTGOING;
          else if (!strcmp("BOTHWAY", buf))
                cfg.circuitType   = ISUP_CIR_BOTHWAY;
          else
                fprintf(stderr, "Warning: CIRCUIT %d unknown circuit type [%s]"
                             "ignored, defaulting to INCOMING\n", circuit, buf);
        }

        if ((next = getcfgval("CONTROL_TYPE", CFGT_STRING, buf, last)) > 0)
        {
            if (!strcmp("NONE", buf))
                cfg.controlType   = ISUP_CTL_NONE;
            else if (!strcmp("ALL", buf))
                cfg.controlType   = ISUP_CTL_ALL;
            else if (!strcmp("ODD_EVEN", buf))
                cfg.controlType   = ISUP_CTL_ODDEVEN;
            else
                fprintf(stderr, "Warning: CIRCUIT %d unknown control type [%s]"
                             "ignored, defaulting to NONE\n", circuit, buf);
        }

        if ((next = getcfgval("BEARER_PROFILE", CFGT_STRING, buf, last)) > 0)
        {
          if (!strcmp("VOICE", buf))
                cfg.bearerProfile = ISUP_PROF_VOICE;
          else
                fprintf(stderr, "Warning: CIRCUIT %d unknown bearer profile [%s]"
                             "ignored, defaulting to VOICE\n", circuit, buf);
        }

        if ((next = getcfgval("CONT_CHECK", CFGT_BOOL, &temp, last)) > 0)
            cfg.contCheckReq  = (Bool)temp;

        if ((next = getcfgval("NON_SS7", CFGT_BOOL, &temp, last)) > 0)
            cfg.nonSS7Con     = (Bool)temp;

        if (getcfgval("SSF", CFGT_STRING, buf, last))
        {
            if (strcmp(buf, "INTERNATIONAL") == 0)
                cfg.ssf = ISUP_SSF_INTL;
            else if (strcmp(buf, "0") == 0)
                cfg.ssf = ISUP_SSF_INTL;
            else if (strcmp(buf, "1") == 0)
                cfg.ssf = ISUP_SSF_SPARE;
            else if (strcmp(buf, "2") == 0)
                cfg.ssf = ISUP_SSF_NAT;
            else if (strcmp(buf, "3") == 0)
                cfg.ssf = ISUP_SSF_RES;
            else if (strcmp(buf, "NATIONAL") == 0)
                cfg.ssf = ISUP_SSF_NAT;
            else if (strcmp(buf, "RESERVED") == 0)
                cfg.ssf = ISUP_SSF_RES;
            else if (strcmp(buf, "SPARE") == 0)
                cfg.ssf = ISUP_SSF_SPARE;
            else
                fprintf(stderr, "Warning: Circuit %d unknown SSF type [%s]"
                                "ignored, defaulting to NATIONAL\n", circuit, buf);
        }

        if ((next = getcfgval("GROUP_CHARS", CFGT_INT, &temp, last)) > 0)
            cfg.groupChars    = (U8)temp;

        GET_ADDRESS("TRUNK",     cfg.outTrunkGrp);
        GET_TIMER("T4_TIMER",    cfg.t4);
        GET_TIMER("T12_TIMER",   cfg.t12);
        GET_TIMER("T13_TIMER",   cfg.t13);
        GET_TIMER("T14_TIMER",   cfg.t14);
        GET_TIMER("T15_TIMER",   cfg.t15);
        GET_TIMER("T37_TIMER",   cfg.t37);
        GET_TIMER("TPAUSE_TIMER",cfg.tPause);
        GET_TIMER("TVAL_TIMER",  cfg.tVal);

#ifdef CFGTEST
        printf("Circuit Configuration\n");
#endif
        while (cirCnt)
        {
            cfg.circuitId = circuit;
            cfg.cic = cic;

#ifdef CFGTEST
            showHex((unsigned char *)&cfg, sizeof(cfg));
#else

            /* Call the Config Request */
            status = isupCircCfg(boardNum, &cfg);
            if (status == CPI_TIMEOUT)
            {
                printf("ISUP Circuit Config: No response from board\n");
                return(status);
            }
            else if (status != ISUP_MGMT_SUCCESS)
            {
                printf("ISUP Circuit Config: Error %d\n", status);
                return(status);
            }

#endif                               /* ! CFGTEST */
            ++circuit;
            ++cic;
            --cirCnt;
        } /* end of while */

   		/* skip up to end of section */
   		next = getcfgval("END", CFGT_INT, &temp, last);
  		last = next;
    }   /* end for loop */
    return(0);

} /* isupCircCfg */

/*
* isupRteCfg
*
* Configure Routing Table
*/
short RteCfg(void)
{
   IsupRouteCfg   cfg;
   unsigned short rtecnt;
   short          status;
   S32            last, next;

    for(last = 0, rtecnt = 0; ;)
	{
		memset(&cfg, 0, sizeof(cfg));

        if ((next = scanRoute(&cfg, last)) <= 0)
        	break;                          /* no more Routes */

    	last = next;                        /* save starting line in config file */
        if (++rtecnt > MaxRoutes)
		{
            fprintf(stderr, "Route %s exceeds maximum [%d]\n",
                                cfg.address, MaxRoutes);
            return(-1);
    	}

#ifdef CFGTEST
        printf("Route Configuration\n");
        showHex((unsigned char *)&cfg, sizeof(cfg));
#else

   		/* send the config request */
        status = isupRouteCfg(boardNum, &cfg);
        if (status == CPI_TIMEOUT)
        {
            printf("ISUP Route Config: No response from board\n");
            return(status);
        }
        else if (status != ISUP_MGMT_SUCCESS)
        {
            printf("ISUP Route Config: Error %d\n", status);
            return(status);
        }

#endif                               /* ! CFGTEST */

    	/* skip up to end of section */
/*
 *     	next = getcfgval("END", CFGT_INT, &temp, last);
    	last = next;
*/
	}   /* end for loop */

   return(0);
} /* isupRteCfg */


/*
*
*  isupcfg
*
*  Configures the ISUP process by calling the individual configuration section
*  scanner routines in the right order.
*/
void isupcfg(unsigned char board)
{
  short   status;

  status = GenCfg();               /* General Configuration */
  if (status != 0)
  {
      return;                         /* fatal */
  }
  printf(" ISUP General configuration successful\n");

  status = SapCfg();               /* User SAPs */
  if (status != 0)
  {
      return;                         /* fatal */
  }
  printf(" ISUP USAP configuration successful\n");

  status = NSapCfg(board);         /* Network Saps */
  if (status != 0)
  {
      return;                      /* fatal */
  }
  printf(" ISUP NSAP configuration successful\n");

  CircCfg();                       /* Circuit Configuration (non-fatal)*/
  RteCfg();                        /* Routing Table Configuration */

} /* end of isupcfg */

/*
 * main
 *
 * parses parameters (board number, config file name) from command line,
 * calls appropriate config routines.
 */
int main(int argc, char *argv[])
{
  int           i;
  char         *p;
  S16           status;
  S16		tmp;
  CDebug = 0;


  printf("%s: ISUP configuration application version %s %s\n",
    argv[0], ISUPCFG_VERSION, __DATE__);

  /* parse command line for config file name & board # */
  for(i = 1; i < argc; i++)
  {
    p = argv[i];
    if (*p == '-') {
      switch (*++p) {
        case 'f' :     /* configuration file name */
          i++;
          if (argv[i] == NULL)
          {
            printf(usage);
            exit(1);
          }

          strncpy(configFile, argv[i], sizeof(configFile) - 1);
          configFile[sizeof(configFile) - 1] = '\0';  /* ensure null term */
          break;

        case 'b' :    /* Set Board number */
          i++;
          if (argv[i] == NULL)
            {
              printf(usage);
              exit(1);
            }

          if ((sscanf(argv[i], "%hi", &tmp) != 1) || (tmp > MAXBOARD))
            {
              printf(usage);
              exit(1);
            }
          else
            boardNum = (U8) tmp;
          break;

        case 'd' :
          debug = 1;              /* i/o packet dumps */
          CDebug = 1;             /* config scanner debugging */
          break;

        default:
          printf(usage);
          exit(1);
          break;
      }	/* end switch */
    }
    else
    {
        printf(usage);
        exit(1);
    }
  } /* end for */

  /* initialize config file scanner library */
  if (initcfg(configFile) != 0)
    {
      exit(1);
    }

  if ((status = isupInitMgmtAPI(boardNum, ISUPCFG_ENT)) != ISUP_MGMT_SUCCESS)
    {
      fprintf(stderr, "Error %d initializing API\n", status);
      exit(1);
    }

  /* do the isup configuration */
  isupcfg(boardNum);

  isupTermMgmtAPI(boardNum);

  endcfg();

  exit(0);
}
