#ifdef WIN32
#include <process.h>
#endif

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#include "osdep.h"
#include "ctadef.h"
#include "ctaerr.h"

#include "ss7type.h"
#include "isupdef.h"

#include "demodef.h"

#define INCST_IDLE          0
#define INCST_WT4ACMEVT     1
#define INCST_WT4ANMEVT     2
#define INCST_CONNECTED     3
#define INCST_WT4RLCMSG     4

#define INC_MAXSTATES       5

static void CallIdle(Circuit *circuit, Event *evt, DWORD evtId);
static void WaitForACMevt(Circuit *circuit, Event *evt, DWORD evtId);
static void WaitForANMevt(Circuit *circuit, Event *evt, DWORD evtId);
static void CallConnected(Circuit *circuit, Event *evt, DWORD evtId);
static void WaitForRLCmsg(Circuit *circuit, Event *evt, DWORD evtId);


StateDesc incStates[INC_MAXSTATES] =
{
    { "CallIdle     ", CallIdle      },
    { "WaitForACMevt", WaitForACMevt },
    { "WaitForANMevt", WaitForANMevt },
    { "CallConnected", CallConnected },
    { "WaitForRLCmsg", WaitForRLCmsg }
};

extern CTAHD    isupHndl;
extern short    traceFlags;
extern S16      switchType;
extern Circuit *circuits;
extern char    *eventNames[];
extern CirId    numIn;
extern CirId    numOut;
extern CirId    firstIn;
extern CirId    lastIn;
extern CirId    firstOut;
extern CirId    lastOut;
extern Bool     multiThread;

void CheckPoint(Circuit *circuit);
CirId CircuitSelect(CirId inCirId, U8 testCall);

void IncomingThread(void *arglist)
{
    Circuit        *circuit = &circuits[(CirId)arglist];
    CTAQUEUEHD      queue;
    CTA_EVENT       ctaEvent;
    DWORD           ret;
    CirId           i;

    circuit->cirId     = (CirId)arglist;
    circuit->mateId    = 0;
    circuit->spInstId  = 0;
    circuit->suInstId  = circuit->cirId;
    circuit->state     = INCST_IDLE;
    circuit->blkState  = BSNOTBLK;
    circuit->callState = CSIDLE;

    if ((ret = ctaCreateQueue(NULL, 0, &queue)) != SUCCESS)
    {
        fprintf(stderr, "Circuit %d IncomongThread() terminating, ctaCreateQueue() failed: %d\n", circuit->cirId, ret);
        THREAD_EXIT();
    }

    if ((ret = ctaCreateContext(queue, 0, NULL, &circuit->ctaHndl)) != SUCCESS)
    {
        fprintf(stderr, "Circuit %d IncomingThread() terminating, ctaCreateContext() failed: %d\n", circuit->cirId, ret);
        THREAD_EXIT();
    }

    if (multiThread != TRUE)
    {
        for (i = (firstIn + 1); i <= lastIn; i++)
        {
            circuits[i].mateId    = 0;
            circuits[i].spInstId  = 0;
            circuits[i].ctaHndl   = circuit->ctaHndl;
            circuits[i].cirId     = i;
            circuits[i].suInstId  = i;
            circuits[i].state     = INCST_IDLE;
            circuits[i].blkState  = BSNOTBLK;
            circuits[i].callState = CSIDLE;
        }
    }
    while (1)
    {
        if ((ret = ctaWaitEvent(queue, &ctaEvent, CTA_WAIT_FOREVER)) != SUCCESS)
        {
            fprintf(stderr, "Circuit %d IncomingThread() terminating, ctaWaitEvent() failed: %d\n", circuit->cirId, ret);
            THREAD_EXIT();
        }
        circuit = &circuits[ctaEvent.value];
        if (traceFlags & EVT_TRACE)
            fprintf(stderr, "IncomingThread: % 4d %s %s \n", circuit->cirId, eventNames[(ctaEvent.id & 0xff)], incStates[circuit->state].stateName);
        (*(incStates[circuit->state].state))(circuit, (Event *)ctaEvent.buffer, ctaEvent.id);
    }
}

static void ThreadError(Circuit *circuit, char *call, DWORD ret)
{
    fprintf(stderr, "Circuit %d IncomingThread() terminating, %s failed: : %d\n", circuit->cirId, call, ret);
    THREAD_EXIT();
}

static void CallIdle(Circuit *circuit, Event *evt, DWORD evtId)
{
    CTA_EVENT       ctaEvent;
    DWORD           ret;
    SiStaEvnt       siEvent = {0};

    memset(&ctaEvent, 0, sizeof(CTA_EVENT));

    switch (evtId)
    {
        case IAM_MSG:
            circuit->spInstId = evt->info.spInstId;
            if (CircuitSelect(circuit->cirId, (U8)(evt->sdu.m.siConEvnt.cgPtyCat.cgPtyCat.val == CAT_TEST)) == 0)
            {
                ISUPInitREL(switchType, &evt->sdu.m.siRelEvnt, CCNOCIRCUIT);
                ISUPReleaseReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, circuit->cirId, &evt->sdu.m.siRelEvnt);
                circuit->mateId    = 0;
                circuit->spInstId  = 0;
                circuit->transient = TRUE;
                CheckPoint(circuit);
                circuit->state     = INCST_WT4RLCMSG;
                free(evt);
                break;
            }
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, IAM_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);

            circuit->state     = INCST_WT4ACMEVT;
            circuit->transient = TRUE;
            CheckPoint(circuit);
            break;

        case REL_MSG:
            ISUPReleaseResp(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, circuit->cirId, &evt->sdu.m.siRelEvnt);
            free(evt);
            break;

        case REL_EVT:
            free(evt);
            break;

        case RLC_MSG:
            free(evt);
            break;

        case RSC_EVT:
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->state    = INCST_WT4RLCMSG;
            circuit->transient = TRUE;
            circuit->blkState &= ~BSLOCBLK;
            CheckPoint(circuit);
            break;

        case RSC_MSG:
            circuit->blkState &= ~BSRMTBLK;
            CheckPoint(circuit);
            free(evt);
            break;

        case BLO_MSG:
            circuit->blkState |= BSRMTBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRBLKRSP, &siEvent);
            CheckPoint(circuit);
            free(evt);
            break;

        case UBL_MSG:
            circuit->blkState &= ~BSRMTBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRUNBLKRSP, &siEvent);
            CheckPoint(circuit);
            free(evt);
            break;

        case BLO_EVT:
            circuit->blkState |= BSLOCBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRBLKREQ, &siEvent);
            CheckPoint(circuit);
            break;

        case UBL_EVT:
            circuit->blkState &= ~BSLOCBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRUNBLKREQ, &siEvent);
            CheckPoint(circuit);
            break;

        case BLA_MSG:
            free(evt);
            break;

        case UBA_MSG:
            free(evt);
            break;

        case GRS_EVT:
            break;

        case CGB_MSG:
            circuit->blkState |= BSRMTBLK;
            CheckPoint(circuit);
            break;

        case CGU_MSG:
            circuit->blkState &= ~BSRMTBLK;
            CheckPoint(circuit);
            break;

        case CGB_EVT:
            circuit->blkState |= BSLOCBLK;
            CheckPoint(circuit);
            break;

        case CGU_EVT:
            circuit->blkState &= ~BSLOCBLK;
            CheckPoint(circuit);
            break;

        case CHKPNT_EVT:
        {
            ChkPntMsg      *chkPntMsg = (ChkPntMsg *)&evt->chkPntMsg;
            SiStaEvnt      *staEvnt   = (SiStaEvnt *)&evt->sdu.m.siStaEvnt;

            staEvnt->rangStat.eh.pres       = PRESENT;
            staEvnt->rangStat.range.pres    = PRESENT;
            staEvnt->rangStat.range.val     = 0;
            staEvnt->rangStat.status.pres   = PRESENT;
            staEvnt->rangStat.status.len    = 1;
            staEvnt->rangStat.status.val[0] = chkPntMsg->blkState | chkPntMsg->callState;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRGRPSET, staEvnt);

            circuit->mateId    = chkPntMsg->mateId;
            circuit->transient = chkPntMsg->transient;
            circuit->blkState  = chkPntMsg->blkState;
            circuit->callState = chkPntMsg->callState;
            if (chkPntMsg->callState == CSINCBUSY)
                circuit->state = INCST_CONNECTED;

            free(evt);
            break;
        }

        case IDLE_EVT:
        {
            SiStaEvnt      staEvnt   = {0};

            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->transient = FALSE;
            circuit->state     = INCST_IDLE;
            circuit->callState = CSIDLE;

            staEvnt.rangStat.eh.pres       = PRESENT;
            staEvnt.rangStat.range.pres    = PRESENT;
            staEvnt.rangStat.range.val     = 0;
            staEvnt.rangStat.status.pres   = PRESENT;
            staEvnt.rangStat.status.len    = 1;
            staEvnt.rangStat.status.val[0] = circuit->blkState | circuit->callState;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRGRPSET, &staEvnt);
            break;
        }

        case ERR_EVT:
            fprintf(stderr, "Error indication received for circuit %d, cause = %d\n", evt->info.circuit, evt->sdu.m.siStaEvnt.causeDgn.causeVal.val);
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->transient = TRUE;
            CheckPoint(circuit);
            circuit->state = INCST_WT4RLCMSG;
            free(evt);
            break;

        default:
            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in CallIdle state, resetting\n", circuit->cirId, evtId);
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->transient = TRUE;
            CheckPoint(circuit);
            circuit->state = INCST_WT4RLCMSG;
            free(evt);
            break;
    }
}

static void WaitForACMevt(Circuit *circuit, Event *evt, DWORD evtId)
{
    CTA_EVENT       ctaEvent;
    DWORD           ret;
    SiStaEvnt       siEvent = {0};

    memset(&ctaEvent, 0, sizeof(CTA_EVENT));

    switch (evtId)
    {
        case ACM_EVT:
            ISUPConnectStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, circuit->cirId, &evt->sdu.m.siCnStEvnt, ADDRCMPLT);
            circuit->state = INCST_WT4ANMEVT;
            free(evt);
            break;

        case REL_MSG:
            ISUPReleaseResp(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, circuit->cirId, &evt->sdu.m.siRelEvnt);
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, REL_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->transient = FALSE;
            CheckPoint(circuit);
            break;

        case REL_EVT:
            ISUPReleaseReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, circuit->cirId, &evt->sdu.m.siRelEvnt);
            circuit->mateId    = 0;
            circuit->state     = INCST_WT4RLCMSG;
            free(evt);
            break;

        case RSC_EVT:
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->mateId    = 0;
            circuit->state     = INCST_WT4RLCMSG;
            circuit->blkState &= ~BSLOCBLK;
            CheckPoint(circuit);
            break;

        case RSC_MSG:
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->transient = FALSE;
            circuit->blkState &= ~BSRMTBLK;
            CheckPoint(circuit);
            free(evt);
            break;

        case BLO_MSG:
            circuit->blkState |= BSRMTBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRBLKRSP, &siEvent);
            CheckPoint(circuit);
            free(evt);
            break;

        case UBL_MSG:
            circuit->blkState &= ~BSRMTBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRUNBLKRSP, &siEvent);
            CheckPoint(circuit);
            free(evt);
            break;

        case BLO_EVT:
            circuit->blkState |= BSLOCBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRBLKREQ, &siEvent);
            CheckPoint(circuit);
            break;

        case UBL_EVT:
            circuit->blkState &= ~BSLOCBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRUNBLKREQ, &siEvent);
            CheckPoint(circuit);
            break;

        case BLA_MSG:
            free(evt);
            break;

        case UBA_MSG:
            free(evt);
            break;

        case GRS_EVT:
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->transient = FALSE;
            CheckPoint(circuit);
            break;

        case CGB_MSG:
            circuit->blkState |= BSRMTBLK;
            CheckPoint(circuit);
            break;

        case CGU_MSG:
            circuit->blkState &= ~BSRMTBLK;
            CheckPoint(circuit);
            break;

        case CGB_EVT:
            circuit->blkState |= BSLOCBLK;
            CheckPoint(circuit);
            break;

        case CGU_EVT:
            circuit->blkState &= ~BSLOCBLK;
            CheckPoint(circuit);
            break;

        case IDLE_EVT:
        {
            SiStaEvnt      staEvnt   = {0};

            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->transient = FALSE;
            circuit->state     = INCST_IDLE;
            circuit->callState = CSIDLE;

            staEvnt.rangStat.eh.pres       = PRESENT;
            staEvnt.rangStat.range.pres    = PRESENT;
            staEvnt.rangStat.range.val     = 0;
            staEvnt.rangStat.status.pres   = PRESENT;
            staEvnt.rangStat.status.len    = 1;
            staEvnt.rangStat.status.val[0] = circuit->blkState | circuit->callState;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRGRPSET, &staEvnt);
            break;
        }

        case CHKPNT_EVT:
            free(evt);
            break;

        case ERR_EVT:
            fprintf(stderr, "Error indication received for circuit %d, cause = %d\n", evt->info.circuit, evt->sdu.m.siStaEvnt.causeDgn.causeVal.val);
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state = INCST_WT4RLCMSG;
            free(evt);
            break;

        default:
            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in WaitForACMevt state, resetting\n", circuit->cirId, evtId);
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state = INCST_WT4RLCMSG;
            free(evt);
            break;
    }
}

static void WaitForANMevt(Circuit *circuit, Event *evt, DWORD evtId)
{
    CTA_EVENT       ctaEvent;
    DWORD           ret;
    SiStaEvnt       siEvent = {0};

    memset(&ctaEvent, 0, sizeof(CTA_EVENT));

    switch (evtId)
    {
        case ANM_EVT:
            ISUPConnectResp(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, circuit->cirId, &evt->sdu.m.siConEvnt);
            circuit->callState = CSINCBUSY;
            circuit->state     = INCST_CONNECTED;
            circuit->transient = FALSE;
            CheckPoint(circuit);
            free(evt);
            break;

        case REL_MSG:
            ISUPReleaseResp(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, circuit->cirId, &evt->sdu.m.siRelEvnt);
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, REL_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->transient = FALSE;
            CheckPoint(circuit);
            break;

        case REL_EVT:
            ISUPReleaseReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, circuit->cirId, &evt->sdu.m.siRelEvnt);
            circuit->mateId    = 0;
            circuit->state     = INCST_WT4RLCMSG;
            free(evt);
            break;

        case RSC_EVT:
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->mateId    = 0;
            circuit->state     = INCST_WT4RLCMSG;
            circuit->blkState &= ~BSLOCBLK;
            CheckPoint(circuit);
            break;

        case RSC_MSG:
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->transient = FALSE;
            circuit->blkState &= ~BSRMTBLK;
            CheckPoint(circuit);
            free(evt);
            break;

        case BLO_MSG:
            circuit->blkState |= BSRMTBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRBLKRSP, &siEvent);
            CheckPoint(circuit);
            free(evt);
            break;

        case UBL_MSG:
            circuit->blkState &= ~BSRMTBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRUNBLKRSP, &siEvent);
            CheckPoint(circuit);
            free(evt);
            break;

        case BLO_EVT:
            circuit->blkState |= BSLOCBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRBLKREQ, &siEvent);
            CheckPoint(circuit);
            break;

        case UBL_EVT:
            circuit->blkState &= ~BSLOCBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRUNBLKREQ, &siEvent);
            CheckPoint(circuit);
            break;

        case BLA_MSG:
            free(evt);
            break;

        case UBA_MSG:
            free(evt);
            break;

        case GRS_EVT:
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->transient = FALSE;
            CheckPoint(circuit);
            break;

        case CGB_MSG:
            circuit->blkState |= BSRMTBLK;
            CheckPoint(circuit);
            break;

        case CGU_MSG:
            circuit->blkState &= ~BSRMTBLK;
            CheckPoint(circuit);
            break;

        case CGB_EVT:
            circuit->blkState |= BSLOCBLK;
            CheckPoint(circuit);
            break;

        case CGU_EVT:
            circuit->blkState &= ~BSLOCBLK;
            CheckPoint(circuit);
            break;

        case IDLE_EVT:
        {
            SiStaEvnt      staEvnt   = {0};

            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->transient = FALSE;
            circuit->state     = INCST_IDLE;
            circuit->callState = CSIDLE;

            staEvnt.rangStat.eh.pres       = PRESENT;
            staEvnt.rangStat.range.pres    = PRESENT;
            staEvnt.rangStat.range.val     = 0;
            staEvnt.rangStat.status.pres   = PRESENT;
            staEvnt.rangStat.status.len    = 1;
            staEvnt.rangStat.status.val[0] = circuit->blkState | circuit->callState;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRGRPSET, &staEvnt);
            break;
        }

        case CHKPNT_EVT:
            free(evt);
            break;

        case ERR_EVT:
            fprintf(stderr, "Error indication received for circuit %d, cause = %d\n", evt->info.circuit, evt->sdu.m.siStaEvnt.causeDgn.causeVal.val);
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state = INCST_WT4RLCMSG;
            free(evt);
            break;

        default:
            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in WaitForANMevt state, resetting\n", circuit->cirId, evtId);
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state = INCST_WT4RLCMSG;
            free(evt);
            break;
    }
}

static void CallConnected(Circuit *circuit, Event *evt, DWORD evtId)
{
    CTA_EVENT       ctaEvent;
    DWORD           ret;
    SiStaEvnt       siEvent = {0};

    memset(&ctaEvent, 0, sizeof(CTA_EVENT));

    switch (evtId)
    {
        case REL_MSG:
            ISUPReleaseResp(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, circuit->cirId, &evt->sdu.m.siRelEvnt);
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, REL_EVT, evt)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->callState = CSIDLE;
            CheckPoint(circuit);
            break;

        case REL_EVT:
            ISUPReleaseReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, circuit->cirId, &evt->sdu.m.siRelEvnt);
            circuit->mateId    = 0;
            circuit->state     = INCST_WT4RLCMSG;
            circuit->callState = CSIDLE;
            circuit->transient = TRUE;
            CheckPoint(circuit);
            free(evt);
            break;

        case RSC_EVT:
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->mateId    = 0;
            circuit->state     = INCST_WT4RLCMSG;
            circuit->callState = CSIDLE;
            circuit->transient = TRUE;
            circuit->blkState &= ~BSLOCBLK;
            CheckPoint(circuit);
            break;

        case RSC_MSG:
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->callState = CSIDLE;
            circuit->state     = INCST_IDLE;
            circuit->blkState &= ~BSRMTBLK;
            CheckPoint(circuit);
            free(evt);
            break;

        case BLO_MSG:
            circuit->blkState |= BSRMTBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRBLKRSP, &siEvent);
            CheckPoint(circuit);
            free(evt);
            break;

        case UBL_MSG:
            circuit->blkState &= ~BSRMTBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRUNBLKRSP, &siEvent);
            CheckPoint(circuit);
            free(evt);
            break;

        case BLO_EVT:
            circuit->blkState |= BSLOCBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRBLKREQ, &siEvent);
            CheckPoint(circuit);
            break;

        case UBL_EVT:
            circuit->blkState &= ~BSLOCBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRUNBLKREQ, &siEvent);
            CheckPoint(circuit);
            break;

        case BLA_MSG:
            free(evt);
            break;

        case UBA_MSG:
            free(evt);
            break;

        case GRS_EVT:
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->callState = CSIDLE;
            circuit->state     = INCST_IDLE;
            CheckPoint(circuit);
            break;

        case CGB_MSG:
            circuit->blkState |= BSRMTBLK;
            CheckPoint(circuit);
            break;

        case CGU_MSG:
            circuit->blkState &= ~BSRMTBLK;
            CheckPoint(circuit);
            break;

        case CGB_EVT:
            circuit->blkState |= BSLOCBLK;
            CheckPoint(circuit);
            break;

        case CGU_EVT:
            circuit->blkState &= ~BSLOCBLK;
            CheckPoint(circuit);
            break;

        case CHKPNT_EVT:
        {
            ChkPntMsg      *chkPntMsg = (ChkPntMsg *)&evt->chkPntMsg;
            SiStaEvnt      *staEvnt   = (SiStaEvnt *)&evt->sdu.m.siStaEvnt;

            staEvnt->rangStat.eh.pres       = PRESENT;
            staEvnt->rangStat.range.pres    = PRESENT;
            staEvnt->rangStat.range.val     = 0;
            staEvnt->rangStat.status.pres   = PRESENT;
            staEvnt->rangStat.status.len    = 1;
            staEvnt->rangStat.status.val[0] = chkPntMsg->blkState | chkPntMsg->callState;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRGRPSET, staEvnt);

            circuit->spInstId  = 0;
            circuit->mateId    = chkPntMsg->mateId;
            circuit->transient = chkPntMsg->transient;
            circuit->blkState  = chkPntMsg->blkState;
            circuit->callState = chkPntMsg->callState;
            if (chkPntMsg->callState == CSIDLE)
                circuit->state = INCST_IDLE;

            free(evt);
            break;
        }

        case IDLE_EVT:
        {
            SiStaEvnt      staEvnt   = {0};

            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->callState = CSIDLE;

            staEvnt.rangStat.eh.pres       = PRESENT;
            staEvnt.rangStat.range.pres    = PRESENT;
            staEvnt.rangStat.range.val     = 0;
            staEvnt.rangStat.status.pres   = PRESENT;
            staEvnt.rangStat.status.len    = 1;
            staEvnt.rangStat.status.val[0] = circuit->blkState | circuit->callState;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRGRPSET, &staEvnt);
            break;
        }

        case ERR_EVT:
            fprintf(stderr, "Error indication received for circuit %d, cause = %d\n", evt->info.circuit, evt->sdu.m.siStaEvnt.causeDgn.causeVal.val);
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->callState = CSIDLE;
            circuit->transient = TRUE;
            CheckPoint(circuit);
            circuit->state = INCST_WT4RLCMSG;
            free(evt);
            break;

        default:
            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in CallConnected state, resetting\n", circuit->cirId, evtId);
            if ((ret = EventCircuit(circuit->cirId, circuit->mateId, RSC_EVT, NULL)) != SUCCESS)
                ThreadError(circuit, "EventCircuit()", ret);
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->callState = CSIDLE;
            circuit->transient = TRUE;
            CheckPoint(circuit);
            circuit->state = INCST_WT4RLCMSG;
            free(evt);
            break;
    }
}

static void WaitForRLCmsg(Circuit *circuit, Event *evt, DWORD evtId)
{
    CTA_EVENT       ctaEvent;
    SiStaEvnt       siEvent = {0};

    memset(&ctaEvent, 0, sizeof(CTA_EVENT));

    switch (evtId)
    {
        case RLC_MSG:
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->transient = FALSE;
            CheckPoint(circuit);
            free(evt);
            break;

        case REL_MSG:
            ISUPReleaseResp(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, circuit->cirId, &evt->sdu.m.siRelEvnt);
/*
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->transient = FALSE;
*/
            free(evt);
            break;

        case RSC_EVT:
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            circuit->blkState &= ~BSLOCBLK;
            CheckPoint(circuit);
            break;

        case RSC_MSG:
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->transient = FALSE;
            circuit->blkState &= ~BSRMTBLK;
            CheckPoint(circuit);
            free(evt);
            break;

        case BLO_MSG:
            circuit->blkState |= BSRMTBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRBLKRSP, &siEvent);
            CheckPoint(circuit);
            free(evt);
            break;

        case UBL_MSG:
            circuit->blkState &= ~BSRMTBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRUNBLKRSP, &siEvent);
            CheckPoint(circuit);
            free(evt);
            break;

        case BLO_EVT:
            circuit->blkState |= BSLOCBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRBLKREQ, &siEvent);
            CheckPoint(circuit);
            break;

        case UBL_EVT:
            circuit->blkState &= ~BSLOCBLK;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRUNBLKREQ, &siEvent);
            CheckPoint(circuit);
            break;

        case BLA_MSG:
            free(evt);
            break;

        case UBA_MSG:
            free(evt);
            break;

        case GRS_EVT:
            circuit->spInstId  = 0;
            circuit->state     = INCST_IDLE;
            circuit->transient = FALSE;
            CheckPoint(circuit);
            break;

        case CGB_MSG:
            circuit->blkState |= BSRMTBLK;
            CheckPoint(circuit);
            break;

        case CGU_MSG:
            circuit->blkState &= ~BSRMTBLK;
            CheckPoint(circuit);
            break;

        case CGB_EVT:
            circuit->blkState |= BSLOCBLK;
            CheckPoint(circuit);
            break;

        case CGU_EVT:
            circuit->blkState &= ~BSLOCBLK;
            CheckPoint(circuit);
            break;

        case IDLE_EVT:
        {
            SiStaEvnt      staEvnt   = {0};

            circuit->mateId    = 0;
            circuit->spInstId  = 0;
            circuit->transient = FALSE;
            circuit->state     = INCST_IDLE;
            circuit->callState = CSIDLE;

            staEvnt.rangStat.eh.pres       = PRESENT;
            staEvnt.rangStat.range.pres    = PRESENT;
            staEvnt.rangStat.range.val     = 0;
            staEvnt.rangStat.status.pres   = PRESENT;
            staEvnt.rangStat.status.len    = 1;
            staEvnt.rangStat.status.val[0] = circuit->blkState | circuit->callState;
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRGRPSET, &staEvnt);
            break;
        }

        case CHKPNT_EVT:
            free(evt);
            break;

        case ERR_EVT:
            fprintf(stderr, "Error indication received for circuit %d, cause = %d\n", evt->info.circuit, evt->sdu.m.siStaEvnt.causeDgn.causeVal.val);
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            free(evt);
            break;

        default:
            fprintf(stderr, "Circuit %d: Invailid event [0x%x] received in WaitForRLCmsg state\n", circuit->cirId, evtId);
            ISUPStatusReq(isupHndl, circuit->suId, circuit->suInstId, circuit->spInstId, FALSE, circuit->cirId, CIRRESREQ, &siEvent);
            free(evt);
            break;
    }
}

