#ifdef WIN32
#include <process.h>
/*#include <winsock.h>*/
#endif

#include <stddef.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#include "osdep.h"
#include "ss7type.h"
#include "isupdef.h"
#include "isupmgmt.h"
#include "demodef.h"

#if defined (__sun) || defined (LINUX)
#define Sleep(x) usleep(1000 * x)
#endif
#ifdef UNIXWARE
#define Sleep(x) sleep(x)
#endif


CTA_SERVICE_NAME  demoServiceNames[] = {{"ISUP", "ISUPMGR"}, { "HMI", "HMIMGR" } };

S16             switchType  = ST_ANS92;     /*  ST_ITUBLUE; ST_ITUWTITE; ST_ANS88; ST_ANS92; ST_ANS95; ST_Q767;  */
U8              boardNum    = 1;
U16             locPort     = 4096;
U16             remPort     = 4096;
U32             remAddr;

Bool            multiThread = FALSE;
int             standAlone  = 0;            /*  by default, not in standalone mode */

short           traceFlags  = 0;
CirId           numIn       = 64;
CirId           numOut      = 64;
CirId           firstIn;
CirId           lastIn;
CirId           firstOut;
CirId           lastOut;
Circuit        *circuits;
TMutex          criticalSection;

extern char    *promptStr;

static S16 GetSapSwType( U8 board, U8 usap );
static char *SwTypeName( S16 swType );

void ISUPThread (void *dummy);
void HMIThread (void *dummy);
void UDPThread (void *dummy);
void UserThread (void *dummy);
void TestThread (void *dummy);
void OutgoingThread(void *arglist);
void IncomingThread(void *arglist);

int main (int argc, char *argv[])
{
    short           optind = 1;
    char           *str;
    struct hostent *hostent;        /* ptr to struct returned by gethostbyname() */
    DWORD           ret;
    char            buf[80];
    CTA_INIT_PARMS  demoInitparms = {0};
    U8              inHigh = FALSE;
    CirId           i;
    TThrId	    threadId;

    /* Initialize remote address to our loopback address */
    remAddr = htonl(INADDR_LOOPBACK);

    while (str = argv[optind])
    {
        if (*str == '-')
        {
            optind++;
            switch (*++str)
            {
                case 'b':   /* which board should we speak to ? */
                    if (*++str)         /* was there no space ? */
                        boardNum = atoi(str);
                    else
                        boardNum = atoi(argv[optind++]);
                    break;

                case 'd':
                    ++str;
                    if (*str == 'p')
                    {
                        if (*++str)         /* was there no space ? */
                            remPort = htons((U16)atoi(str));
                        else
                            remPort = htons((U16)atoi(argv[optind++]));
                    }
                    else if (*str == 'a')
                    {
                        if (*++str)         /* was there no space ? */
                            ;               /* str is OK now */
                        else
                            str = argv[optind++];
                        remAddr = inet_addr(str);
                    }
                    else if (*str == 'n')
                    {
                        if (*++str)         /* was there no space ? */
                            ;               /* str is OK now */
                        else
                            str = argv[optind++];
                        if ((hostent = gethostbyname(str)) == NULL)
                        {
                            fprintf(stderr,  "UDPThread terminating,  invalid mate IP address/host name [%s]\n", str);
                            THREAD_EXIT();
                        }

                        /* got host info for far end, use 1st IP address */
                        memcpy((char *)&(remAddr), hostent->h_addr, hostent->h_length);
                    }
                    break;

                case 'h':               /* run without HMI, must be standalone case */
                    standAlone = 1;
                    promptStr = "Stand Alone> ";
                    break;

                case 'i':
                    inHigh = TRUE;
                    break;

                case 'l':
                    ++str;
                    if (*str == 'p')
                    {
                        if (*++str)         /* was there no space ? */
                            locPort = htons((U16)atoi(str));
                        else
                            locPort = htons((U16)atoi(argv[optind++]));
                    }
                    break;

                case 'm':
                    multiThread = TRUE;
                    break;

                case 'n':   /* which board should we speak to ? */
                    ++str;
                    if (*str == '\0')
                        numIn = numOut = atoi(argv[optind++]);
                    else if (*str == 'i')
                    {
                        ++str;
                        if (*str == '\0')
                            numIn = atoi(argv[optind++]);
                        else
                            numIn = atoi(str);
                    }
                    else if (*str == 'o')
                    {
                        ++str;
                        if (*str == '\0')
                            numOut = atoi(argv[optind++]);
                        else
                            numOut = atoi(str);
                    }
                    else
                        numIn = numOut = atoi(str);
                    break;

                case 't':   /* which board should we speak to ? */
                    ++str;
                    if (*str == 'e')
                        traceFlags |= EVT_TRACE;
                    if (*str == 'c')
                        traceFlags |= CHKPNT_TRACE;
                    if (*str == 'a')
                        traceFlags = (short)0xffff;
                    break;

                default:
                    break;
            }
        }
        else
            break;
    }

    /* determine switch type from management API */
    switchType = GetSapSwType( boardNum, SAP_ID );
    printf( "Board %d Sap %d switch type = %s [%d]\n",
            boardNum, SAP_ID, SwTypeName( switchType ), switchType );

    if (inHigh == TRUE) /* incoming circuits use higher circuit IDs */
    {
        firstIn  = numOut + 1;
        lastIn   = numOut + numIn;
        firstOut = 1;
        lastOut  = numOut;
    }
    else
    {
        firstIn  = 1;
        lastIn   = numIn;
        firstOut = numIn + 1;
        lastOut  = numIn + numOut;
    }

    if ((circuits = (Circuit *)calloc(((numIn + numOut) + 1), sizeof(Circuit))) == NULL)
    {
        fprintf(stderr, "Could not allocate circuit control blocks - Exiting\n");
        exit(1);
    }

    /* Initialize CT Access health management (HMI) service */
    demoInitparms.size = sizeof(CTA_INIT_PARMS);
    demoInitparms.ctacompatlevel = CTA_COMPATLEVEL;

    MUTEX_CREATE(criticalSection);

    if ((ret = ctaInitialize(demoServiceNames, 2, &demoInitparms)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        fprintf(stderr, "HMIThread terminating, error initializing CT Access: %s\n", buf);
        THREAD_EXIT();
    }

    THREAD_CREATE(0, (TPtrThrFunc) ISUPThread, argv, 0, &threadId);
    THREAD_CREATE(0, (TPtrThrFunc) UserThread, argv, 0, &threadId);
    THREAD_CREATE(0, (TPtrThrFunc) TestThread, &circuits[0], 0, &threadId);

    if( !standAlone )
    {
        /* HMI in use, create HMI thread and UDP thread for checkpointing */
        THREAD_CREATE(0, (TPtrThrFunc) UDPThread, argv, 0, &threadId);
        THREAD_CREATE(0, (TPtrThrFunc) HMIThread, argv, 0, &threadId);
    }

    if (multiThread)
    {
        for (i = firstIn; i <= lastIn; i++)
        {
            circuits[i].cirId = i;
	    THREAD_CREATE(0, (TPtrThrFunc) IncomingThread, (void *) i, 0, &threadId);
        }
        for (i = firstOut; i <= lastOut; i++)
        {
            circuits[i].cirId = i;
	    THREAD_CREATE(0, (TPtrThrFunc) OutgoingThread, (void *) i, 0, &threadId);
        }
    }
    else
    {
      THREAD_CREATE(0, (TPtrThrFunc) IncomingThread, (void *) firstIn, 0, &threadId);
      THREAD_CREATE(0, (TPtrThrFunc) OutgoingThread, (void *) firstOut, 0, &threadId);
    }
    while (1)
        Sleep(1000);
}


CirId CircuitSelect(CirId inCirId, U8 testCall)
{
    static char     init    = 0;
    static CirId    nextOut;
    CirId           cirIdx;

    if (!init)
    {
        init = 1;
        nextOut = firstOut;
    }

    if (testCall == TRUE)
        return(1);

    MUTEX_LOCK(criticalSection);

    for (cirIdx = nextOut; cirIdx <= lastOut; cirIdx++)
    {
        if ((circuits[cirIdx].callState == CSIDLE)   &&
            (circuits[cirIdx].transient == FALSE)    &&
            !circuits[cirIdx].blkState)
        {
            if (cirIdx == lastOut)
                nextOut = firstOut;
            else
                nextOut = (cirIdx + 1);
            circuits[inCirId].mateId = cirIdx;
            circuits[cirIdx].mateId  = inCirId;
/*            fprintf(stderr, "Circuit %d selected\n", cirIdx);*/
	    MUTEX_UNLOCK(criticalSection);
            return(cirIdx);
        }
    }

    for (cirIdx = nextOut = firstOut; cirIdx <= lastOut; cirIdx++)
    {
        if ((circuits[cirIdx].callState == CSIDLE)   &&
            (circuits[cirIdx].transient == FALSE)    &&
            !circuits[cirIdx].blkState)
        {
            if (cirIdx == lastOut)
                nextOut = firstOut;
            else
                nextOut = (cirIdx + 1);
            circuits[inCirId].mateId = cirIdx;
            circuits[cirIdx].mateId  = inCirId;
/*            fprintf(stderr, "Circuit %d selected\n", cirIdx);*/
	    MUTEX_UNLOCK(criticalSection);
            return(cirIdx);
        }
    }

    MUTEX_UNLOCK(criticalSection);
    return((CirId)0);
}

/* SwTypeName
*
* This function returns a printable name for a switch type (protocol variant).
*
*/
static char *SwTypeName( S16 swType )
{
    switch( swType )
    {
        case ST_ITUWHITE:   return( "ITU-WHITE" );
        case ST_ANS88:      return( "ANSI-88" );
        case ST_ANS92:      return( "ANSI-92" );
        case ST_ITUBLUE:    return( "ITU-BLUE" );
        case ST_ANS95:      return( "ANSI-95" );
        case ST_Q767:       return( "Q-767" );
        case ST_JNTT:       return( "J-NTT" );
        default:            return( "Unknown" );
	}       /* of switch */

} /* end of SwTypeName */


/* GetSapSwType
 *
 * Determines the ISUP switch type (protocol variant) of the specified board/user SAP
 * by opening the ISUP management API for that board and retrieving the user SAP
 * configuration.
 *
 * Parameters:
 *  board       board number to open
 *  usap        user SAP number
 *
 * Returns:
 *  switch type read from ISUP config; if couldn't be read, defaults to ANSI-92.
 */
static S16 GetSapSwType( U8 board, U8 usap )
{
    S16             swType  = ST_ANS92;
    S16             status;
    IsupUSapCfg     usapcfg;

    /* open the ISUP management API and fetch the switch type from SAP config */
    status = isupInitMgmtAPI( board, MGMT_ENT_ID );
    if( status != ISUP_MGMT_SUCCESS )
    {
        printf( "Error: isupInitMgmtAPI [board %d] failed [%d], defaulting to ANSI-92\n", board, status );
        return( swType );
    }

    /* save entire SAP config for use by scripts */
    status = isupGetUSapCfg( board, &usapcfg, usap );
    if( status != ISUP_MGMT_SUCCESS )
    {
        printf( "Error: isupGetUSapCfg [board %d, SAP %d] failed [%d], defaulting to ANSI-92\n",
                board, usap, status );
    }
    else
    {
        swType = usapcfg.switchType;
    }

    isupTermMgmtAPI( board );
    return( swType );
}                                       /* GetSapSwType */
