/*
 * term.c - sample call termination application
 *
 */

#include <string.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef UNIX
#include <ctype.h>
#endif	                                /* UNIX */

#include "osdep.h"
#include "ss7type.h"
#include "isupdef.h"
#include "demodef.h"

void GroupReq(CirId cirId, SiRangStat *rangStat, int evtId);
DWORD EventCircuit(CirId srcId, CirId dstId, DWORD evtId, Event *event);

/* globals */
extern U32 lastTrans;
extern U32 lastIncCnt;
extern U32 lastOutCnt;

char           *promptStr = "Unknown> ";
extern U16      switchType;
extern CirId    numIn;
extern CirId    numOut;
extern CirId    firstIn;
extern CirId    lastIn;
extern CirId    firstOut;
extern CirId    lastOut;
extern Circuit *circuits;
extern CTAHD    isupHndl;

/* Define Instance and Entity IDs required of any application that
 * utilized the ISUP API.
 */

void cmdHELP(char *args[])
{
    if (args[0] == NULL)
    {
            printf("QUIT        exit application\n");
            printf("ACM         send an address complete message\n");
            printf("ANM         send an answer message\n");
            printf("BLO         send a blocking message\n");
            printf("CCR         send a continuity check request message\n");
            printf("CGB         send a circuit group blocking message\n");
            printf("CGU         send a circuit group unblocking message\n");
            printf("CON         send a connect message\n");
            printf("COT         send a continuity report message\n");
            printf("CPG         send a call progress message\n");
            printf("CQM         send a circuit group query message\n");
            printf("CRA         send a circuit reservation acknowledgement message\n");
            printf("CRM         send a circuit reservation message\n");
            printf("FOT         send a forward transfer message\n");
            printf("GRS         send a circuit group reset message\n");
            printf("IAM         send an initial address message\n");
            printf("LPA         send a loopback acknowledgement message\n");
            printf("REL         send a release message\n");
            printf("RES         send a resume message\n");
            printf("RLC         send a release complete message\n");
            printf("RSC         send a circuit reset message\n");
            printf("SAM         send an subsequent address message\n");
            printf("SUS         send a suspend message\n");
            printf("UBL         send an unblocking message\n");
            printf("AUTO        set automatic response mode\n");
    }
    else
    {
        if (strcmp(args[0], "Q") == 0)
            exit(0);
        if (strcmp(args[0], "QUIT") == 0)
            exit(0);
        if (strcmp(args[0], "ACM") == 0)
            printf("Syntax: ACM <circuid ID>\n");
        if (strcmp(args[0], "ANM") == 0)
            printf("Syntax: ANM <circuid ID>\n");
        if (strcmp(args[0], "BLO") == 0)
            printf("Syntax: BLO <circuid ID>\n");
        if (strcmp(args[0], "CCR") == 0)
            printf("Syntax: CCR <circuid ID>\n");
        if (strcmp(args[0], "CGB") == 0)
            printf("Syntax: CGB <circuid ID> <range>\n");
        if (strcmp(args[0], "CGU") == 0)
            printf("Syntax: CGU <circuid ID> <range>\n");
        if (strcmp(args[0], "CON") == 0)
            printf("Syntax: CON <circuid ID>\n");
        if (strcmp(args[0], "COT") == 0)
            printf("Syntax: COT <circuid ID>\n");
        if (strcmp(args[0], "CPG") == 0)
            printf("Syntax: CPG <circuid ID>\n");
        if (strcmp(args[0], "CQM") == 0)
            printf("Syntax: CQM <circuid ID> <range>\n");
        if (strcmp(args[0], "CRA") == 0)
            printf("Syntax: CRA <circuid ID>\n");
        if (strcmp(args[0], "CRM") == 0)
            printf("Syntax: CRM <circuid ID> [c]\n");
        if (strcmp(args[0], "FOT") == 0)
            printf("Syntax: FOT <circuid ID>\n");
        if (strcmp(args[0], "GRS") == 0)
            printf("Syntax: GRS <circuid ID> <range>\n");
        if (strcmp(args[0], "IAM") == 0)
            printf("Syntax: IAM <circuid ID> <called> [<calling>]\n");
        if (strcmp(args[0], "LPA") == 0)
            printf("Syntax: LPA <circuid ID>\n");
        if (strcmp(args[0], "REL") == 0)
            printf("Syntax: REL <circuid ID>[<cause>]\n");
        if (strcmp(args[0], "RES") == 0)
            printf("Syntax: RES <circuid ID>\n");
        if (strcmp(args[0], "RLC") == 0)
            printf("Syntax: RLC <circuid ID>\n");
        if (strcmp(args[0], "RSC") == 0)
            printf("Syntax: RSC <circuid ID>\n");
        if (strcmp(args[0], "SAM") == 0)
            printf("Syntax: SAM <circuid ID>  <subnumber>\n");
        if (strcmp(args[0], "SUS") == 0)
            printf("Syntax: SUS <circuid ID>\n");
        if (strcmp(args[0], "UBL") == 0)
            printf("Syntax: UBL <circuid ID>\n");
        if (strcmp(args[0], "CIRCUIT") == 0)
            printf("Syntax: CIRCUIT <circuit ID>\n");
    }
}

void cmdACM(char *args[])
{
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    ISUPInitACM( switchType, &evt->sdu.m.siCnStEvnt);

    if ((ret = EventCircuit(0, cirId, ACM_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}

void cmdANM(char *args[])
{
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    ISUPInitANM( switchType, &evt->sdu.m.siConEvnt);

    if ((ret = EventCircuit(0, cirId, ANM_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}

void cmdBLO(char *args[])
{
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    char        buf[80];

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((ret = EventCircuit(0, cirId, BLO_EVT, NULL)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}

void cmdCCR(char *args[])
{
    printf("UserThread: Not yet supported\n");
    return;
}

void cmdCGB(char *args[])
{
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    Event      *evt;
    int         i;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    evt->sdu.m.siStaEvnt.rangStat.eh.pres    = PRESENT;
    evt->sdu.m.siStaEvnt.rangStat.range.pres = PRESENT;
    evt->sdu.m.siStaEvnt.rangStat.range.val  = atoi(args[1]);

    if (switchType == ST_ITUWHITE || switchType == ST_ITUBLUE || switchType == ST_Q767)
    {
        if (evt->sdu.m.siStaEvnt.rangStat.range.val > 31)
        {
                printf("Error: range number must be [0-31]\n");
                return;
        }
    }
    else
        if (evt->sdu.m.siStaEvnt.rangStat.range.val > 23)
        {
                printf("Error: range number must be [0-23]\n");
                return;
        }


    evt->sdu.m.siStaEvnt.cgsmti.eh.pres      = PRESENT;
    evt->sdu.m.siStaEvnt.cgsmti.typeInd.pres = PRESENT;

    if (args[2] != NULL)
        evt->sdu.m.siStaEvnt.cgsmti.typeInd.val = atoi(args[2]);
    else
        evt->sdu.m.siStaEvnt.cgsmti.typeInd.val = 0;

    evt->sdu.m.siStaEvnt.rangStat.status.pres = PRESENT;
    evt->sdu.m.siStaEvnt.rangStat.status.len  = (evt->sdu.m.siStaEvnt.rangStat.range.val / 8) + 1;

    for(i = 0; i <= evt->sdu.m.siStaEvnt.rangStat.range.val; i++)
        evt->sdu.m.siStaEvnt.rangStat.status.val[i / 8] |= 1 << (i % 8);

    ret = ISUPStatusReq(isupHndl, circuits[cirId].suId, 0, 0, 0, cirId, CIRGRPBLKREQ, &evt->sdu.m.siStaEvnt);

    if (ret != ISUP_SUCCESS)
        printf("UserThread: Error sending Curcuit Group Block request, status = %d\n", ret);

    GroupReq(cirId, &evt->sdu.m.siStaEvnt.rangStat, CGB_EVT);
    return;
}

void cmdCGU(char *args[])
{
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    Event      *evt;
    int         i;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    evt->sdu.m.siStaEvnt.rangStat.eh.pres    = PRESENT;
    evt->sdu.m.siStaEvnt.rangStat.range.pres = PRESENT;
    evt->sdu.m.siStaEvnt.rangStat.range.val  = atoi(args[1]);

    if (switchType == ST_ITUWHITE || switchType == ST_ITUBLUE || switchType == ST_Q767)
    {
        if (evt->sdu.m.siStaEvnt.rangStat.range.val > 31)
        {
                printf("Error: range number must be [0-31]\n");
                return;
        }
    }
    else
        if (evt->sdu.m.siStaEvnt.rangStat.range.val > 23)
        {
                printf("Error: range number must be [0-31]\n");
                return;
        }


    evt->sdu.m.siStaEvnt.cgsmti.eh.pres      = PRESENT;
    evt->sdu.m.siStaEvnt.cgsmti.typeInd.pres = PRESENT;

    if (args[2] != NULL)
        evt->sdu.m.siStaEvnt.cgsmti.typeInd.val = atoi(args[2]);
    else
        evt->sdu.m.siStaEvnt.cgsmti.typeInd.val = 0;

    evt->sdu.m.siStaEvnt.rangStat.status.pres = PRESENT;
    evt->sdu.m.siStaEvnt.rangStat.status.len  = (evt->sdu.m.siStaEvnt.rangStat.range.val / 8) + 1;

    for(i = 0; i <= evt->sdu.m.siStaEvnt.rangStat.range.val; i++)
        evt->sdu.m.siStaEvnt.rangStat.status.val[i / 8] |= 1 << (i % 8);

    ret = ISUPStatusReq(isupHndl, circuits[cirId].suId, 0, 0, 0, cirId, CIRGRPUNBLKREQ, &evt->sdu.m.siStaEvnt);

    if (ret != ISUP_SUCCESS)
        printf("UserThread: Error sending Curcuit Group UnBlock request, status = %d\n", ret);

    GroupReq(cirId, &evt->sdu.m.siStaEvnt.rangStat, CGU_EVT);
    return;
}

void cmdCON(char *args[])
{
    printf("UserThread: Not yet supported\n");
    return;
}

void cmdCOT(char *args[])
{
    printf("UserThread: Not yet supported\n");
    return;
}

void cmdCPG(char *args[])
{
    printf("UserThread: Not yet supported\n");
    return;
}

void cmdCQM(char *args[])
{
    printf("UserThread: Not yet supported\n");
    return;
}

void cmdCRA(char *args[])
{
    printf("UserThread: Not yet supported\n");
    return;
}

void cmdCRM(char *args[])
{
    printf("UserThread: Not yet supported\n");
    return;
}

void cmdFOT(char *args[])
{
    printf("UserThread: Not yet supported\n");
    return;
}

void cmdGRS(char *args[])
{
    printf("UserThread: Not yet supported\n");
    return;
}

void cmdIAM(char *args[])
{
    U8          cdPty[20];
    U8          cgPty[20];
    U8          cdPtyLen;
    U8          cgPtyLen;
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) < firstOut) || (cirId > lastOut))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    if (args[1] == NULL)
    {
        printf("UserThread: Missing called number\n");
        return;
    }

    cdPtyLen = ISUPASCIItoBCD((U8 *)args[1], cdPty, 20);

    cgPtyLen = ISUPASCIItoBCD((U8 *)args[2], cgPty, 20);

    ISUPInitIAM( switchType,
                &evt->sdu.m.siConEvnt,
                 cdPty,
                 cdPtyLen,
                 cgPty,
                 cgPtyLen);

    evt->sdu.m.siConEvnt.cgPtyCat.cgPtyCat.val = CAT_TEST;

    if ((ret = EventCircuit(0, cirId, IAM_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}

void cmdLPA(char *args[])
{
    printf("UserThread: Not yet supported\n");
    return;
}

void cmdREL(char *args[])
{
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    char        buf[80];
    Event      *evt;
    U8          causeVal;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    if (args[1] == NULL)
        causeVal = CCCALLCLR;
    else
        causeVal = (U8)atoi(args[1]);

    ISUPInitREL( switchType,
                &evt->sdu.m.siRelEvnt,
                 causeVal);

    if ((ret = EventCircuit(0, cirId, REL_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}

void cmdRES(char *args[])
{
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    ISUPInitRES( switchType, &evt->sdu.m.siResmEvnt);

    if ((ret = EventCircuit(0, cirId, RES_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}

void cmdRLC(char *args[])
{
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    if ((ret = EventCircuit(0, cirId, RLC_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}

void cmdRSC(char *args[])
{
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    char        buf[80];

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((ret = EventCircuit(0, cirId, RSC_EVT, NULL)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}

void cmdSAM(char *args[])
{
    printf("UserThread: Not yet supported\n");
    return;
}

void cmdStats(char *args[])
{
    U32 currTrans = 0;
    U32 currInc   = 0;
    U32 currOut   = 0;
    CirId   i;

    for (i = 1; i <= (numIn + numOut); i++)
    {
        if (circuits[i].transient)
            ++currTrans;
        if ( circuits[i].callState == CSINCBUSY)
            ++currInc;
        if ( circuits[i].callState == CSOUTBUSY)
            ++currOut;
    }

    printf("\nlastTrans = %4d, lastIncCnt = %4d, lastOutCnt = %4d\ncurrTrans = %4d, currInc = %4d, currOut = %4d\n",
              lastTrans,       lastIncCnt,       lastOutCnt,       currTrans,       currInc,       currOut);
    return;
}

void cmdSUS(char *args[])
{
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    char        buf[80];
    Event      *evt;

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((evt = (Event *)calloc(1, sizeof(Event))) == NULL)
    {
        printf("UserThread: Error allocating event\n");
        return;
    }

    ISUPInitSUS( switchType, &evt->sdu.m.siSuspEvnt);

    if ((ret = EventCircuit(0, cirId, SUS_EVT, evt)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}

void cmdUBL(char *args[])
{
    DWORD       ret;
    CirId       cirId;
    CTA_EVENT   ctaEvent = {0};
    char        buf[80];

    if (args[0] == NULL)
    {
        printf("UserThread: Missing circuit ID\n");
        return;
    }
    if (((cirId = strtoul(args[0], NULL, 10)) == 0L) || (cirId > (numIn + numOut)))
    {
        printf("UserThread: Invalid circuit ID\n");
        return;
    }

    if ((ret = EventCircuit(0, cirId, UBL_EVT, NULL)) != SUCCESS)
    {
        ctaGetText(NULL_CTAHD, ret, buf, sizeof(buf));
        printf("UserThread: EventCircuit returned error, %s\n", buf);
        return;
    }
}

void strtoupper(char *s)
{
    while (*s != '\0')
        *s++ = toupper(*s);
}

#define MAX_ARGS    16
#define SEPCHARS    " \n"

void UserThread(void *dummy)
{
    static char in[133];
    static char *inPtr = in;
    char *cmd;
    char *args[MAX_ARGS + 1];
    int i;

    while (1)
    {
        printf(promptStr);
        fgets(in, 132, stdin);
        while ( (strlen(in) > 0) && (in[strlen(in) - 1] < ' ') )
            in[strlen(in) - 1] = '\0';      /* remove any trailing non-printable characters */

        strtoupper(in);
        if ((cmd = strtok(in, SEPCHARS)) == NULL)
            continue;

        memset( args,
                0,
                ((sizeof(char *) * MAX_ARGS) + 1));

        for (i = 0; i <= MAX_ARGS; i++)
        {
            if ((args[i] = strtok(NULL, SEPCHARS)) == NULL)
                break;
        }

        if (strcmp(cmd, "Q") == 0)
            exit(0);
        if (strcmp(cmd, "QUIT") == 0)
            exit(0);
        else if (strcmp(cmd, "ACM") == 0)
            cmdACM(args);
        else if (strcmp(cmd, "ANM") == 0)
            cmdANM(args);
        else if (strcmp(cmd, "BLO") == 0)
            cmdBLO(args);
        else if (strcmp(cmd, "CCR") == 0)
            cmdCCR(args);
        else if (strcmp(cmd, "CGB") == 0)
            cmdCGB(args);
        else if (strcmp(cmd, "CGU") == 0)
            cmdCGU(args);
        else if (strcmp(cmd, "CON") == 0)
            cmdCON(args);
        else if (strcmp(cmd, "COT") == 0)
            cmdCOT(args);
        else if (strcmp(cmd, "CPG") == 0)
            cmdCPG(args);
        else if (strcmp(cmd, "CQM") == 0)
            cmdCQM(args);
        else if (strcmp(cmd, "CRA") == 0)
            cmdCRA(args);
        else if (strcmp(cmd, "CRM") == 0)
            cmdCRM(args);
        else if (strcmp(cmd, "FOT") == 0)
            cmdFOT(args);
        else if (strcmp(cmd, "GRS") == 0)
            cmdGRS(args);
        else if (strcmp(cmd, "IAM") == 0)
            cmdIAM(args);
        else if (strcmp(cmd, "LPA") == 0)
            cmdLPA(args);
        else if (strcmp(cmd, "REL") == 0)
            cmdREL(args);
        else if (strcmp(cmd, "RES") == 0)
            cmdRES(args);
        else if (strcmp(cmd, "RLC") == 0)
            cmdRLC(args);
        else if (strcmp(cmd, "RSC") == 0)
            cmdRSC(args);
        else if (strcmp(cmd, "SAM") == 0)
            cmdSAM(args);
        else if (strcmp(cmd, "SUS") == 0)
            cmdSUS(args);
        else if (strcmp(cmd, "UBL") == 0)
            cmdUBL(args);
        else if (strcmp(cmd, "STATS") == 0)
            cmdStats(args);
        else if (strcmp(cmd, "HELP") == 0)
            cmdHELP(args);
        else if (strcmp(cmd, "?") == 0)
            cmdHELP(args);
        else
            printf("Error: Unrecognized command, type HELP for list of commands\n");
    }
}

