/*************************************************************************
**                                                                      **
**                         PROPRIETARY NOTICE                           **
**                                                                      **
**                         Copyright (c) 1996                           **
**                    by Natural MicroSystems, Inc.                     **
**                                                                      **
**  The following is licensed program material and is considered        **
**  proprietary and confidential.  No part of this program may be       **
**  reproduced or used in any form or by any means without the express  **
**  written permission of Natural MicroSystems, Inc.                    **
**                                                                      **
*************************************************************************/

/* ISUP management test application */

#ifdef UNIX
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#endif                                  /* UNIX */

#ifdef WIN32
#include <stdio.h>
#include <conio.h>
#include <stdlib.h>
#include <string.h>
#endif                                  /* WIN32 */

#ifdef __OS2__
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#endif                                  /* __OS2__ */

#include "ss7type.h"
#include "isupmgmt.h"
#include "isupver.h"
#include "txcpi.h"

void validateCircuit( short *parms );
void resetCircuit( short *parms );
void blockCircuit( short *parms );
void unblockCircuit( short *parms );
void deleteCircuit( short *parms );
void circuitStatus( short *parms );
void statsReq( short *parms );
void traceReq( short *parms );
void HelpCommand( short *parms );
void setBoard( short *parms );
void getConfigReq( short *parms );
void quit( short *parms );

char *callState [] =
{
    "transient",
    "continuity check",
    "unknown",
    "unknown",
    "incoming busy",
    "unknown",
    "unknown",
    "unknown",
    "outgoing busy",
    "unknown",
    "unknown",
    "unknown",
    "idle",
    "unknown",
    "unknown",
    "unknown",
};

#define MAXCALLSTATE    0x0f

char *transState [] =
{
    "idle",
    "unequipped",
    "locally blocked",
    "waiting for blocking ack",
    "waiting for unblocking ack",
    "locally blocked, waiting for blocking response",
    "locally blocked, waiting for unblocking response",
    "remotely blocked, waiting for blocking ack",
    "remotely blocked, waiting for unblocking ack",
    "locally and remotely blocked",
    "waiting for blocking ack and blocking response",
    "waiting for unblocking ack and unblocking response",
    "waiting for blocking ack and unblocking response",
    "waiting for unblocking ack and blocking response",
    "waiting for blocking response",
    "waiting for unblocking response",
    "remotely blocked"
};

#define MAXTRANSSTATE    0x10

/* lookup tables for translating names into values and vice versa */
struct lookupEnt
{
	char 	*name;
	short	value;
};


/* parameter name tables */

struct lookupEnt StatsLookupTbl[] =
{
    { "NSAP",    0 },
    { "CIRCUIT", 1 },
    { 0,         0 }
};

struct lookupEnt GetConfigLookupTbl[] =
{
    { "GEN",     1 },
    { "USAP",    2 },
    { "NSAP",    3 },
    { "CIRCUIT", 4 },
    { 0,         0 }
};

struct lookupEnt OnOffLookupTbl[] =
{
    { "ON",  1 },
    { "OFF", 0 },
    { 0,     0 }
};

struct lookupEnt TraceLookupTbl[] =
{
    { "EVENT",   EVENT_TRACE   },
    { "DATA",    TRACE_DATA    },
    { "ERROR",   ENCODE_ERROR  },
    { "WARNING", ENUM_WARNING  },
    { "ELEMENT", INIT_ELMT_DBG },
    { "TOKEN",   INIT_TKN_DBG  },
    { "TIMER",   TMR_TRACE     },
    { 0,         0             }
};

struct lookupEnt CmdLookupTbl[] =
{
        { "VALIDATE",   1  },
        { "RESET",      2  },
        { "BLOCK",      3  },
        { "UNBLOCK",    4  },
        { "DELETE",     11 },
        { "STATUS",     5  },
        { "STATS",      6  },
        { "GET",        7  },
        { "TRACE",      8  },
        { "BOARD",      9  },
        { "Q",          10 },
        { "ALL",        0  }
};

static char validateHelp[]        =   "    Syntax: VALIDATE <number> \n\n"
"     where:\n"
"            <number>      Circuit ID\n\n";

static char getConfigHelp[]        =   "    Syntax: GET <entity> [<number>] \n\n"
"     where:\n"
"            <entity>      [GEN | USAP | NSAP | CIRCUIT]\n"
"            <number>      Circuit or USAP ID\n\n";

static char resetHelp[]        =   "    Syntax: RESET <number>\n\n"
"     where:\n"
"            <number>      Circuit ID\n\n";

static char blockHelp[]        =   "    Syntax: BLOCK <number>\n\n"
"     where:\n"
"            <number>      Circuit ID\n\n";

static char unblockHelp[]        =   "    Syntax: UNBLOCK <number>\n\n"
"     where:\n"
"            <number>      Circuit ID\n\n";

static char deleteHelp[]        =   "    Syntax: DELETE <number>\n\n"
"     where:\n"
"            <number>      Circuit ID\n\n";

static char statusHelp[]        =   "    Syntax: STATUS <number>\n\n"
"     where:\n"
"            <number>      Circuit ID\n\n";

static char statsHelp[]        =   "    Syntax: STATS <entity> <number> <reset>\n\n"
"     where:\n"
"            <entity>      [NSAP | CIRCUIT]\n"
"            <number>      NSAP or Circuit ID\n\n"
"            <reset>       0 (do not reset) or 1 (do reset) \n\n";

static char quitHelp[]              =   "    Syntax: Q \n\n"
"    Quit isupmgr application\n\n\n";

static char traceHelp[]               =   "    Syntax: TRACE {EVENT | DATA | ERROR | WARNING | ELEMENT | TOKEN | TIMER} {ON | OFF}\n\n"
"    Sets tracing level\n\n";

static char boardHelp[]        =   "    Syntax: BOARD <new target board number>\n\n";

static char allHelp[]               =
"    VALIDATE   Validate circuit\n"
"    RESET      Reset circuit\n"
"    BLOCK      Block circuit\n"
"    UNBLOCK    Unblock circuit\n"
"    DELETE     Delete circuit\n"
"    STATUS     Circuit status\n"
"    STATS      Circuit/NSAP statistics\n"
"    GET        Gets configuration information\n"
"    TRACE      Enable/disable tracing\n"
"    BOARD      Changes target board\n"
"    Q          Quit isupmgr application\n\n";

static char* helpTbl[] =
{
      allHelp,
      validateHelp,
      resetHelp,
      blockHelp,
      unblockHelp,
      statusHelp,
      statsHelp,
      getConfigHelp,
      traceHelp,
      boardHelp,
      quitHelp,
      deleteHelp
};
/* structure for command parse tables */
#define MAX_PARMS	10

struct CmdSpec
{
	char				*command;
	struct lookupEnt	*parmTbl[MAX_PARMS];
	void				(*func)(short *);
};

/* Help Command */
struct CmdSpec  helpCmd =
{
      "?",               /* command name */
	    { CmdLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
	    HelpCommand         /* handler function */
};

/* isupmgmt API commands */
struct CmdSpec  validateCmd =
{
    "VALIDATE",             /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },
    validateCircuit            /* handler function */
};

struct CmdSpec  resetCmd =
{
    "RESET",                              /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },    /* parameter lookups */
    resetCircuit                             /* handler function */
};

struct CmdSpec  blockCmd =
{
    "BLOCK",                             /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },   /* parameter lookups */
    blockCircuit                            /* handler function */
};

struct CmdSpec  unblockCmd =
{
    "UNBLOCK",                               /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },  /* parameter lookups */
    unblockCircuit                                /* handler function */
};

struct CmdSpec  deleteCmd =
{
    "DELETE",                               /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },  /* parameter lookups */
    deleteCircuit                                /* handler function */
};

struct CmdSpec	statusCmd =
{
    "STATUS",               /* command name */
    { 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 }, /* parameter lookups */
    circuitStatus                           /* handler function */
};

struct CmdSpec	statsCmd =
{
    "STATS",                /* command name */
    { StatsLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },  /* parameter lookups */
    statsReq              /* handler function */
};

struct CmdSpec  getConfigCmd =
{
    "GET",                            /* command name */
    { GetConfigLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0, 0 },  /* parameter lookups */
    getConfigReq                            /* handler function */
};

struct CmdSpec	boardCmd =
{
    "BOARD",                      /* command name */
    {  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },        /* parameter lookups */
    setBoard                                           /* handler function */
};

struct CmdSpec	quitCmd =
{
    "Q",                        /* command name */
		{  0, 0, 0, 0, 0, 0, 0, 0, 0, 0 },				/* parameter lookups */
		quit											/* handler function */
};

struct CmdSpec  traceCmd =
{
    "TRACE",                                            /* command name */
    {  TraceLookupTbl, OnOffLookupTbl, 0, 0, 0, 0, 0, 0, 0, 0 },              /* parameter lookups */
    traceReq                                            /* handler function */
};

struct CmdSpec *CmdTable[] = {
        &validateCmd,
        &resetCmd,
        &blockCmd,
        &unblockCmd,
        &deleteCmd,
        &statusCmd,
        &statsCmd,
        &boardCmd,
        &quitCmd,
        &helpCmd,
        &traceCmd,
        &getConfigCmd,
        0
};


/* Misc. globals */
char	valStr[20];
unsigned char   board = 1;
unsigned char   channel = ISUPMGR_ENT;
short	quittinTime = 0;

/* getname
 *
 * Serches lookup table for a value match and returns a pointer to
 * the name that corresponds to that value.  Returns a pointer to a
 * global string containing ascii representation of that value if
 * no matching value found in table.
 */
char *getname( struct lookupEnt *tbl, short val )
{
	while( tbl->name != 0 )
	{		/* while not end of table */
		if( tbl->value == val )
		{
			/* match found */
			return( tbl->name );
		}
		tbl++;
	}	/* end while */

	/* falling through loop means search failed - convert number to ascii */
	sprintf( valStr, "%d", val );
	return( valStr );

}	/* end getname() */


/* getval
 *
 * Serches lookup table for a string match and generates the resulting
 * value if found.  Returns 1 if string found, 0 if not found.
 */
short getval( struct lookupEnt *tbl, char *s, short *val )
{
	while( tbl->name != 0 )
	{		/* while not end of table */
		if( strcmp( tbl->name, s ) == 0 )
		{
			/* match found */
			*val = tbl->value;
			return( 1 );
		}
		tbl++;
	}	/* end while */

	/* falling through loop means search failed */
	return( 0 );
}	/* end getval() */

void printBool(char *label, Bool bool)
{
    printf("%s%s\n", label, ((bool) ? "True" : "False"));
}

void printAddr(char *label, IsupAddr *addr)
{
    if (addr->length)
        printf("%s%*s\n", label, addr->length, addr->string);
    else
        printf("%sNot Present\n", label);
}

void printTimer(char *label, TimerCfg *timer)
{
    printf("%s% 3d, %s\n", label, timer->value,((timer->enable) ? "Enabled" : "Disabled"));
}

void printPC(char *label, U32 pc)
{
    printf("%s%d.%d.%d\n", label, ((pc >> 16) & 0xff), ((pc >> 8) & 0xff), (pc & 0xff));
}

void traceReq( short *parms )
{

    S16     status;

    if ((status = isupTraceControl(board, (Bool)parms[1], (U32)parms[0])) != ISUP_MGMT_SUCCESS)
        printf( "Board %d Trace Control Request failed: status = %d\n",
                board, parms[0], status );
}


/*
 * linkCtrl command handler; command syntax:
 *  	LINK <link number>  ENA | DIS | INH | UNI | RST
 */
void validateCircuit( short *parms )
{

    S16     status;

    if ((status = isupValidateCircuit(board, parms[0])) != ISUP_MGMT_SUCCESS)
        printf( "Board %d Circuit %d Validate Request failed: status = %d\n",
                board, parms[0], status );
}

void resetCircuit( short *parms )
{
    S16     status;

    if ((status = isupResetCircuit(board, parms[0])) != ISUP_MGMT_SUCCESS)
        printf( "Board %d Circuit %d Reset Request failed: status = %d\n",
                board, parms[0], status );
}

void blockCircuit( short *parms )
{
    S16     status;

    if ((status = isupBlockCircuit(board, parms[0])) != ISUP_MGMT_SUCCESS)
        printf( "Board %d Circuit %d Block Request failed: status = %d\n",
                board, parms[0], status );
}

void unblockCircuit( short *parms )
{
    S16     status;

    if ((status = isupUnblockCircuit(board, parms[0])) != ISUP_MGMT_SUCCESS)
        printf( "Board %d Circuit %d Unblock Request failed: status = %d\n",
                board, parms[0], status );
}

void deleteCircuit( short *parms )
{
    S16     status;

    if ((status = isupDeleteCircuit(board, parms[0])) != ISUP_MGMT_SUCCESS)
        printf( "Board %d Circuit %d Delete Request failed: status = %d\n",
                board, parms[0], status );
}

/*
 * statusReq command handler
 */
void circuitStatus( short *parms )
{
    S16             status;
    IsupCircStatus  sts;

    if ((status = isupCircuitStatus(board, parms[0], &sts)) != ISUP_MGMT_SUCCESS)
        printf( "Board %d Circuit %d Status Request failed: status = %d\n",
                board, parms[0], status );
    else
        printf("Circuit %d :\n\t callState: %s \n\t transState: %s\n\n",
                parms[0],
               ((sts.callState > MAXCALLSTATE) ? "unknown" : callState[sts.callState]),
               ((sts.transState > MAXTRANSSTATE) ? "unknown" : transState[sts.transState]));
}

/*
 * statsReq command handler
 */
void statsReq( short *parms )
{
    S16             status;

    switch (parms[0])
    {
        case 0:                         /* NSAP */
        {
            IsupNSapStats sts;

            if ((status = isupNSapStatsEx(board, parms[1], &sts, (U8) parms[2])) != ISUP_MGMT_SUCCESS)
            {
                printf( "Board %d NSAP %d Statistics Request failed: status = %d\n",
                        board, parms[1], status );
                return;
            }
            printf("\n");
            printf("adrCmpltTx   : % 8ld   ", sts.adrCmpltTx);
            printf("answerTx     : % 8ld   ", sts.answerTx);
            printf("progressTx   : % 8ld   ", sts.progressTx);
            printf("\n");
            printf("contiTx      : % 8ld   ", sts.contiTx);
            printf("conChkReqTx  : % 8ld   ", sts.conChkReqTx);
            printf("loopBckAckTx : % 8ld   ", sts.loopBckAckTx);
            printf("\n");
            printf("confusTx     : % 8ld   ", sts.confusTx);
            printf("callModReqTx : % 8ld   ", sts.callModReqTx);
            printf("callModRejTx : % 8ld   ", sts.callModRejTx);
            printf("\n");
            printf("callModComTx : % 8ld   ", sts.callModComTx);
            printf("suspTx       : % 8ld   ", sts.suspTx);
            printf("resmTx       : % 8ld   ", sts.resmTx);
            printf("\n");
            printf("forwTx       : % 8ld   ", sts.forwTx);
            printf("conTx        : % 8ld   ", sts.conTx);
            printf("relTx        : % 8ld   ", sts.relTx);
            printf("\n");
            printf("overldTx     : % 8ld   ", sts.overldTx);
            printf("relCmpltTx   : % 8ld   ", sts.relCmpltTx);
            printf("facTx        : % 8ld   ", sts.facTx);
            printf("\n");
            printf("facAckTx     : % 8ld   ", sts.facAckTx);
            printf("facRejTx     : % 8ld   ", sts.facRejTx);
            printf("initAdrTx    : % 8ld   ", sts.initAdrTx);
            printf("\n");
            printf("infoTx       : % 8ld   ", sts.infoTx);
            printf("infoReqTx    : % 8ld   ", sts.infoReqTx);
            printf("passAlongTx  : % 8ld   ", sts.passAlongTx);
            printf("\n");
            printf("subsAdrTx    : % 8ld   ", sts.subsAdrTx);
            printf("usrToUsrTx   : % 8ld   ", sts.usrToUsrTx);
            printf("uneqCirIdTx  : % 8ld   ", sts.uneqCirIdTx);
            printf("\n");
            printf("cirReserveTx : % 8ld   ", sts.cirReserveTx);
            printf("cirResAckTx  : % 8ld   ", sts.cirResAckTx);
            printf("exitTx       : % 8ld   ", sts.exitTx);
            printf("\n");
            printf("netResMgmtTx : % 8ld   ", sts.netResMgmtTx);
            printf("netIdReqTx   : % 8ld   ", sts.netIdReqTx);
            printf("netIdRspTx   : % 8ld   ", sts.netIdRspTx);
            printf("\n");
            printf("\n");
            printf("adrCmpltRx   : % 8ld   ", sts.adrCmpltRx);
            printf("answerRx     : % 8ld   ", sts.answerRx);
            printf("progressRx   : % 8ld   ", sts.progressRx);
            printf("\n");
            printf("contiRx      : % 8ld   ", sts.contiRx);
            printf("conChkReqRx  : % 8ld   ", sts.conChkReqRx);
            printf("loopBckAckRx : % 8ld   ", sts.loopBckAckRx);
            printf("\n");
            printf("confusRx     : % 8ld   ", sts.confusRx);
            printf("callModReqRx : % 8ld   ", sts.callModReqRx);
            printf("callModRejRx : % 8ld   ", sts.callModRejRx);
            printf("\n");
            printf("callModComRx : % 8ld   ", sts.callModComRx);
            printf("suspRx       : % 8ld   ", sts.suspRx);
            printf("resmRx       : % 8ld   ", sts.resmRx);
            printf("\n");
            printf("forwRx       : % 8ld   ", sts.forwRx);
            printf("conRx        : % 8ld   ", sts.conRx);
            printf("overldRx     : % 8ld   ", sts.overldRx);
            printf("\n");
            printf("relRx        : % 8ld   ", sts.relRx);
            printf("relCmpltRx   : % 8ld   ", sts.relCmpltRx);
            printf("facRx        : % 8ld   ", sts.facRx);
            printf("\n");
            printf("facAckRx     : % 8ld   ", sts.facAckRx);
            printf("facRejRx     : % 8ld   ", sts.facRejRx);
            printf("initAdrRx    : % 8ld   ", sts.initAdrRx);
            printf("\n");
            printf("infoReqRx    : % 8ld   ", sts.infoReqRx);
            printf("infoRx       : % 8ld   ", sts.infoRx);
            printf("passAlongRx  : % 8ld   ", sts.passAlongRx);
            printf("\n");
            printf("subsAdrRx    : % 8ld   ", sts.subsAdrRx);
            printf("usrToUsrRx   : % 8ld   ", sts.usrToUsrRx);
            printf("uneqCirIdRx  : % 8ld   ", sts.uneqCirIdRx);
            printf("\n");
            printf("cirReserveRx : % 8ld   ", sts.cirReserveRx);
            printf("cirResAckRx  : % 8ld   ", sts.cirResAckRx);
            printf("exitRx       : % 8ld   ", sts.exitRx);
            printf("\n");
            printf("netResMgmtRx : % 8ld   ", sts.netResMgmtRx);
            printf("netIdReqRx   : % 8ld   ", sts.netIdReqRx);
            printf("netIdRspRx   : % 8ld   ", sts.netIdRspRx);
            printf("\n");
            break;
        }
        case 1:                         /* Circuit */
        {
            IsupCircStats sts;

            if ((status = isupCircuitStatsEx(board, parms[1], &sts, (U8) parms[2])) != ISUP_MGMT_SUCCESS)
            {
                printf( "Board %d Circuit %d Statistics Request failed: status = %d\n",
                        board, parms[0], status );
                return;
            }
            printf("\n");
            printf("blockTx           : % 8ld        ", sts.blockTx );
            printf("blockAckTx        : % 8ld        ", sts.blockAckTx );
            printf("\n");
            printf("unblockTx         : % 8ld        ", sts.unblockTx );
            printf("unblockAckTx      : % 8ld        ", sts.unblockAckTx );
            printf("\n");
            printf("cirResTx          : % 8ld        ", sts.cirResTx );
            printf("cirGrBlockTx      : % 8ld        ", sts.cirGrBlockTx );
            printf("\n");
            printf("cirGrBlockAckTx   : % 8ld        ", sts.cirGrBlockAckTx );
            printf("cirGrUnBlockTx    : % 8ld        ", sts.cirGrUnBlockTx );
            printf("\n");
            printf("cirGrUnBlockAckTx : % 8ld        ", sts.cirGrUnBlockAckTx );
            printf("cirGrQTx          : % 8ld        ", sts.cirGrQTx );
            printf("\n");
            printf("cirGrQAckTx       : % 8ld        ", sts.cirGrQAckTx );
            printf("cirGrResTx        : % 8ld        ", sts.cirGrResTx );
            printf("\n");
            printf("cirGrResAckTx     : % 8ld        ", sts.cirGrResAckTx );
            printf("usrPrtTstTx       : % 8ld        ", sts.usrPrtTstTx );
            printf("\n");
            printf("usrPrtAvTx        : % 8ld        ", sts.usrPrtAvTx );
            printf("\n");
            printf("cirValTstTx       : % 8ld        ", sts.cirValTstTx );
            printf("cirValRspTx       : % 8ld        ", sts.cirValRspTx );
            printf("\n");
            printf("\n");
            printf("blockRx           : % 8ld        ", sts.blockRx );
            printf("blockAckRx        : % 8ld        ", sts.blockAckRx );
            printf("\n");
            printf("unblockRx         : % 8ld        ", sts.unblockRx );
            printf("unblockAckRx      : % 8ld        ", sts.unblockAckRx );
            printf("\n");
            printf("cirResRx          : % 8ld        ", sts.cirResRx );
            printf("cirGrBlockRx      : % 8ld        ", sts.cirGrBlockRx );
            printf("\n");
            printf("cirGrBlockAckRx   : % 8ld        ", sts.cirGrBlockAckRx );
            printf("cirGrUnBlockRx    : % 8ld        ", sts.cirGrUnBlockRx );
            printf("\n");
            printf("cirGrUnBlockAckRx : % 8ld        ", sts.cirGrUnBlockAckRx );
            printf("cirGrQRx          : % 8ld        ", sts.cirGrQRx );
            printf("\n");
            printf("cirGrQAckRx       : % 8ld        ", sts.cirGrQAckRx );
            printf("cirGrResRx        : % 8ld        ", sts.cirGrResRx );
            printf("\n");
            printf("cirGrResAckRx     : % 8ld        ", sts.cirGrResAckRx );
            printf("usrPrtTstRx       : % 8ld        ", sts.usrPrtTstRx );
            printf("\n");
            printf("usrPrtAvRx        : % 8ld        ", sts.usrPrtAvRx );
            printf("\n");
            printf("cirValTstRx       : % 8ld        ", sts.cirValTstRx );
            printf("cirValRspRx       : % 8ld        ", sts.cirValRspRx );
            printf("\n");
            break;
        }
    }

}

/*
 * getConfigReq command handler
 */
void getConfigReq( short *parms )
{
    S16             status;

    switch (parms[0])
    {
        case 1:                         /* General */
        {
            IsupGenParms cfg;

            if ((status = isupGetGenCfg(board, &cfg)) != ISUP_MGMT_SUCCESS)
            {
                printf( "Board %d GENERAL: Get Configuration Request failed: status = %d\n",
                        board, parms[1], status );
                return;
            }
            printf("maxSaps      : %d\n",  cfg.maxSaps);
            printf("maxNetSaps   : %d\n",  cfg.maxNetSaps);
            printf("maxCircuits  : %d\n",  cfg.maxCircuits);
            printf("maxCircGrp   : %d\n",  cfg.maxCircGrp);
            printf("maxCallRefs  : %ld\n", cfg. maxCallRefs);
            printf("maxRoutes    : %d\n",  cfg.maxRoutes);
            printf("txmonTimeout : %d\n",  cfg.txmonTimeout);
            printPC("orgPointCode : ",     cfg.orgPointCode);
            printBool("sccpSupport  : ",   cfg.sccpSupport);
            printAddr("comLLID      : ", &cfg.comLLID);
            printf("poolUpper    : %d\n",  cfg.poolUpper);
            printf("poolLower    : %d\n",  cfg.poolLower);
            printBool("igPassAlng  : ",   cfg.igPassAlng);
            printBool("extElmts    : ",   cfg.extElmts);
            printBool("rawMsgs     : ",   cfg.rawMsgs);
            printBool("oneGrpMsg   : ",   cfg.oneGrpMsg);
            printBool("rmtUsrUnavl : ",   cfg.rmtUsrUnavl);
            printBool("grpResetEvent : ",   cfg.grpResetEvent);
            printBool("slsFromCICs : ",   cfg.slsFromCICs);
            printBool("dsblRmtUsrUnavl : ",   cfg.dsblRmtUsrUnavl);
            printBool("restartT7   : ",   cfg.restartT7);
            printBool("disableACL  : ",   cfg.disableACL);
            printf("timerRes     : %d\n",  cfg.timerRes);
            printTimer("t18          : ", &cfg.t18);
            printTimer("t19          : ", &cfg.t19);
            printTimer("t20          : ", &cfg.t20);
            printTimer("t21          : ", &cfg.t21);
            printTimer("t22          : ", &cfg.t22);
            printTimer("t23          : ", &cfg.t23);
            printTimer("t28          : ", &cfg.t28);
            printTimer("tFrstGrpRx   : ", &cfg.tFrstGrpRx);
            printTimer("tGrpReset    : ", &cfg.tGrpReset);
            printf("qCongOnset1  : %d\n",  cfg.qCongOnset1);
            printf("qCongAbate1  : %d\n",  cfg.qCongAbate1);
            printf("qCongOnset2  : %d\n",  cfg.qCongOnset2);
            printf("qCongAbate2  : %d\n",  cfg.qCongAbate2);
            printf("qCongOnset3  : %d\n",  cfg.qCongOnset3);
            printf("qCongAbate3  : %d\n",  cfg.qCongAbate3);
            printf("mCongOnset1  : %d\n",  cfg.mCongOnset1);
            printf("mCongAbate1  : %d\n",  cfg.mCongAbate1);
            printf("mCongOnset2  : %d\n",  cfg.mCongOnset2);
            printf("mCongAbate2  : %d\n",  cfg.mCongAbate2);
            printf("mCongOnset3  : %d\n",  cfg.mCongOnset3);
            printf("mCongAbate3  : %d\n",  cfg.mCongAbate3);

            break;
        }
        case 2:                         /* USAP */
        {
            IsupUSapCfg cfg;

            if ((status = isupGetUSapCfg(board, &cfg, parms[1])) != ISUP_MGMT_SUCCESS)
            {
                printf( "Board %d USAP %d Get Configuration Request failed: status = %d\n",
                        board, parms[1], status );
                return;
            }
            printf("switchType     : %d\n",  cfg.switchType);
            printBool("wildCardRoute  : ",   cfg.wildCardRoute);
            printf("wildCardMask   : %s\n",  cfg.wildCardMask);
            printBool("sidInsert      : ",   cfg.sidInsert);
            printBool("sidVerify      : ",   cfg.sidVerify);
            printAddr("sid            : ",  &cfg.sid);
            printf("natAddrInd     : %d\n",  cfg.natAddrInd);
            printf("sidNumPlan     : %d\n",  cfg.sidNumPlan);
            printf("sidPresInd     : %d\n",  cfg.sidPresInd);
            printBool("incSidPresRes  : ",   cfg.incSidPresRes);
            printBool("sidPresRes     : ",   cfg.sidPresRes);
            printBool("reqOpt         : ",   cfg.reqOpt);
            printBool("allowCallMod   : ",   cfg.allowCallMod);
            printf("maxMsgLength   : %d\n",  cfg.maxMsgLength);
            printTimer("t1             : ", &cfg.t1);
            printTimer("t2             : ", &cfg.t2);
            printTimer("t5             : ", &cfg.t5);
            printTimer("t6             : ", &cfg.t6);
            printTimer("t7             : ", &cfg.t7);
            printTimer("t8             : ", &cfg.t8);
            printTimer("t9             : ", &cfg.t9);
            printTimer("t31            : ", &cfg.t31);
            printTimer("t33            : ", &cfg.t33);
            printTimer("tCCR           : ", &cfg.tCCR);
            printTimer("t27            : ", &cfg.t27);
            printTimer("t34            : ", &cfg.t34);
            printTimer("tEx            : ", &cfg.tEx);
            printTimer("tCRM           : ", &cfg.tCRM);
            printTimer("tCRA           : ", &cfg.tCRA);
            printf("prior          : %d\n", cfg.prior);
            printf("route          : %d\n", cfg.route);
            printf("selector       : %d\n", cfg.selector);
            break;
        }
        case 3:                         /* NSAP */
        {
            IsupNSapCfg cfg;

            if ((status = isupGetNSapCfg(board, &cfg, parms[1])) != ISUP_MGMT_SUCCESS)
            {
                printf( "Board %d NSAP %d Get Configuration Request failed: status = %d\n",
                        board, parms[1], status );
                return;
            }
            printf("switchType  : %d\n", cfg.switchType);
            printf("spId        : %d\n", cfg.spId);
            printf("ssf         : %d\n", cfg.ssf);
            printf("dstEntity   : %d\n", cfg.dstEntity);
            printf("dstInstance : %d\n", cfg.dstInstance);
            printf("priority    : %d\n", cfg.priority);
            printf("route       : %d\n", cfg.route);
            printf("dstProcId   : %d\n", cfg.dstProcId);
            printf("sapType     : %d\n", cfg.sapType);
            printf("numResInd   : %d\n", cfg.numResInd);
            printf("selector    : %d\n", cfg.selector);
            break;
        }
        case 4:                         /* Circuit */
        {
            IsupCircCfg cfg;
            IsupGenParms cfgGen;

            if ((status = isupGetCircCfg(board, &cfg, (U32)parms[1])) != ISUP_MGMT_SUCCESS)
            {
                printf( "Board %d Circuit %d Get Configuration Request failed: status = %d\n",
                        board, parms[0], status );
                return;
            }
            printf   ("circuitId     : %ld\n", cfg.circuitId);
            printf   ("cic           : %ld\n", cfg.cic);

            isupGetGenCfg(board, &cfgGen);                          /* altOPC */
            if (cfg.altOrgPointCode != cfgGen.orgPointCode)
            {
                printPC("altOrgPntCode : ", cfg.altOrgPointCode);
            }
            else
            {
                printf("altOrgPntCode : DEFAULT ");
                printPC("-> ", cfgGen.orgPointCode);
            }

            printPC  ("dstPointCode  : ",      cfg.dstPointCode);
            printf   ("controlType   : %d\n",  cfg.controlType);
            printf   ("bearerProfile : %d\n",  cfg.bearerProfile);
            printBool("contCheckReq  : ",      cfg.contCheckReq);
            printBool("nonSS7Con     : ",      cfg.nonSS7Con);
            printf   ("ssf           : %d\n",  cfg.ssf);
            printAddr("outTrunkGrp   : ",     &cfg.outTrunkGrp);
            printAddr("comLLID       : ",     &cfg.comLLID);
            printTimer("t4            : ",     &cfg.t4);
            printTimer("t12           : ",     &cfg.t12);
            printTimer("t13           : ",     &cfg.t13);
            printTimer("t14           : ",     &cfg.t14);
            printTimer("t15           : ",     &cfg.t15);
            printTimer("t37           : ",     &cfg.t37);
            printTimer("tPause        : ",     &cfg.tPause);
            printTimer("tVal          : ",     &cfg.tVal);
            break;
        }
    }

}

void HelpCommand( short *parms )
{
	printf( "%s", helpTbl[*parms] );
}

void setBoard( short *parms )
{
    unsigned char       newboard;
    S16                 status;

    newboard = (unsigned char)parms[0];

    if( newboard > MAXBOARD )
	{
        printf( "Board must be in range 0..%d\n", MAXBOARD );
	}

	/* open/init the MTP3 mgmt API for new board*/
    status = isupInitMgmtAPI(newboard, channel);
    if( status != ISUP_MGMT_SUCCESS )
	{
        printf( "Board %d isupInitMgmtAPI failed [%d]\n", newboard, status );
	}
	else
	{
        /* successful, close old board & set new board as the current board */
        status = isupTermMgmtAPI(board);
        board = newboard;
	}
}

void quit( short *parms )
{
	quittinTime = 1;
}


/* processCmd
 *
 * Searches command table for command match, converts parameters to
 * array of shorts, and invokes the appropriate command handler.
 */
void processCmd( char *tokens[], short cnt )
{
	short 				i, j;
	short				found;
	short 				params[MAX_PARMS];	/* numeric parameters array */
	struct lookupEnt	*lp;

	/* first search command table for command match */
	for( i = 0; CmdTable[i] != 0; i++ )
	{
		if( strcmp( tokens[0], CmdTable[i]->command ) == 0 )
		{
			break;		/* command matched */
		}
	}	/* end for */

	/* if no match found display error message and return */
	if( CmdTable[i] == 0 )
	{
		printf("Unrecognized command: %s\n", tokens[0] );
		return;
	}

	/* convert remaining tokens (i.e., parameters) to short integers,
	 * using lookup tables  where specified to convert text strings
	 * (names) into numbers.  Note that all parameters are positional.
	 */
	for( j = 0; j < MAX_PARMS; j++ )
		params[j] = 0;					/* first initialize to defaults */

	for( j = 1; j < cnt; j++, lp++ )
	{
		lp = CmdTable[i]->parmTbl[j - 1];
		if( lp )
		{	/* lookup table for this param ? */
			found = getval( lp, tokens[j], &params[j - 1] );
			if( found )
			{
				continue;				/* parameter succesfully converted */
			}
		}

		/* no lookup table or lookup failed - try to convert as a number */
		if( sscanf( tokens[j], "%hi", &params[j - 1] ) != 1 )
		{
			/* conversion failed */
			printf("Unrecognized parameter: %s\n", tokens[j] );
			return;
		}
	}	/* end for */

	/* call command handler function */

	if( CmdTable[i]->func != 0 )
	{
		CmdTable[i]->func( params );
	}
}

/* tokenize
 *
 * Takes input line and creates an array of pointers to individual
 * tokens in the input line.  First separator after each token in
 * input line is replaced with a null so that each token becomes
 * a null terminated string.
 */
short tokenize( char *buf, char *tokens[] )
{
	char	*p = buf;
	char	*endp;
	short   cnt;

	/* initialize token pointers to all null */
	for( cnt = 0; cnt <= MAX_PARMS; cnt++ )
		tokens[cnt] = 0;

	/* begin scanning for tokens */
	cnt = 0;
	while( *p != 0 && cnt <= MAX_PARMS )
	{
		/* skip leading space or tabs */
		while( *p == ' ' || *p == '\t' )
			p++;

		/* beginning of token (or end of line) */
		if( *p != 0 )
		{
			/* scan for end of token, converting to upper case as we go */
			endp = p;
			while( *endp != ' ' && *endp != '\t' && *endp != 0 )
			{
				if( islower( *endp ) )
				{
					*endp = 'A' + (*endp - 'a');	/* convert to upper */
				}
				endp++;
			}

			/* now have a begin & end of token */
			tokens[cnt++] = p;
			if( *endp == 0 )
			{
				p = endp;	/* end of input - terminate scan */
			}
			else
			{
				*endp = 0;		/* null terminate token */
				p = &endp[1];	/* start next scan at after null */
			}
		}	/* end if *p != 0 */
	}	/* end while *p != 0 .... */

	return( cnt );
}

void usage( void )
{
    printf("\n usage: isupmgr -b [1] \n");
    printf("              -b [Board number]\n");
	exit(1);
}

int main( int argc, char *argv[] )
{

	char	buffer[132];
	char	*tokens[MAX_PARMS +1];
	short	cnt;
	short	i;
	char	*p;
  short   status;
  short   tmp;

    printf("\n\n%s: sample ISUP management application version %s %s\n\n",
            argv[0], ISUPMGR_VERSION, __DATE__);

    /* parse command line for interrupt & board # */
	for(i = 1; i < argc; i++)
	{
		p = argv[i];
		if (*p == '-')
		{
			switch (*++p)
			{
			case 'b' :    /* Set Board number */
        ++p;
        if (*p != 0)
        {
            if ((sscanf(p, "%hi", &tmp) != 1))
                usage();
            board = (unsigned char)tmp;
            if (board > MAXBOARD)
                usage();
        }
        else
        {
            i++;
            if ((sscanf(argv[i], "%hi", &tmp) != 1))
              usage();
            board = (unsigned char)tmp;
            if (board > MAXBOARD)
                usage();
        }
        break;

			default:
				usage();
				break;
			}	/* end switch */
		}
		else
		{
			usage();
		}

	}	/* end for */

	/* open/init the MTP3 mgmt API */
    status = isupInitMgmtAPI(board, channel);
    if( status != ISUP_MGMT_SUCCESS )
	{
        printf( "isupInitMgmtAPI failed [%d]\n", status );
		exit(1);
	}

	/* command processing loop */
    printf( "isupmgr[%-1u]>", board );
    while( (quittinTime == 0) && fgets( buffer, 132, stdin ) != 0 )
	{
        while ( (strlen(buffer) > 0) && (buffer[strlen(buffer) - 1] < ' ') )
            buffer[strlen(buffer) - 1] = '\0';  /* remove any trailing non-printable characters */
		cnt = tokenize( buffer, tokens );
		if( cnt > 0 )
			processCmd( tokens, cnt );
        printf( "isupmgr[%-1u]>", board );
	}	/* end while */

    status = isupTermMgmtAPI(board);
}	/* end main() */
